#include "doc_content.h"

DocContent::DocContent(QWidget *parent) :
    QWidget(parent)
{
    //setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    docLayout = new QGridLayout();
    selinuxLed = new QPushButton(this);
    selinuxLabel = new QLabel("SELinux", this);

    errorLog = new QTextEdit(this);
    errorLog->setToolTip("Error Log");
    errorLog->setReadOnly(true);
    errorLog->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(errorLog, SIGNAL(customContextMenuRequested(const QPoint&)), errorLog, SLOT(clear()));

    docLayout->addWidget(selinuxLed, 1, 0, Qt::AlignLeft);
    docLayout->addWidget(selinuxLabel, 1, 1);
    docLayout->addWidget(errorLog, 2, 0, 3, 2);
    setLayout(docLayout);

    currentSEState.append(QString());
    connect(selinuxLed, SIGNAL(clicked()), this, SLOT(changeSEState()));
    callThread = new CallThread(this);
    connect(callThread, SIGNAL(resultData(QStringList)), this, SLOT(resultReceiver(QStringList)));
}
DocContent::~DocContent()
{
    disconnect(selinuxLed, SIGNAL(clicked()), this, SLOT(changeSEState()));
    disconnect(callThread, SIGNAL(resultData(QStringList)), this, SLOT(resultReceiver(QStringList)));
    disconnect(errorLog, SIGNAL(customContextMenuRequested(const QPoint&)), errorLog, SLOT(clear()));
    if (callThread) callThread->stop();
    delete callThread;
    callThread = 0;
    delete selinuxLed;
    selinuxLed = 0;
    delete selinuxLabel;
    selinuxLabel = 0;
    errorLog->clear();
    delete errorLog;
    errorLog = 0;
    delete docLayout;
    docLayout = 0;
}
void DocContent::setInitContent(QSize s)
{
    _size = s;
}
void DocContent::setLightSEIcon(const QString &state)
{
    if (currentSEState != state) {
        currentSEState = state;
        emit stateChanged(state);
    };
    selinuxLedIcon = QIcon::fromTheme("ledlightred");
    selinuxLed->setIcon(selinuxLedIcon);
    selinuxLed->setToolTip(QString("SELinux is %1").arg(state));
}
void DocContent::setDarkSEIcon(const QString &state)
{
    if (currentSEState != state) {
        currentSEState = state;
        emit stateChanged(state);
    };
    selinuxLedIcon = QIcon::fromTheme("leddarkblue");
    selinuxLed->setIcon(selinuxLedIcon);
    selinuxLed->setToolTip(QString("SELinux is %1").arg(state));
}
QString DocContent::getSELinuxState() const
{
    return currentSEState;
}
void DocContent::changeSEState()
{
    QStringList setMode;
    if (currentSEState == "Enforcing") {
        setMode = QStringList()<<"setCurrentSELinuxMode"<<"0";
    } else {
        setMode = QStringList()<<"setCurrentSELinuxMode"<<"1";
    };
    selinuxLed->setEnabled(false);
    actionActivated(setMode);
}
bool DocContent::actionActivated(const QStringList &args)
{
    if (args.isEmpty()) return false;
    //qDebug() << "pretending to be the mechanism for action:" << args.first();

    Authority::Result result;
    UnixProcessSubject subject(static_cast<uint>(QCoreApplication::applicationPid()));

    result = Authority::instance()->checkAuthorizationSync(QString("%1.%2").arg(DOMAIN).arg(args.first()), subject,
             Authority::AllowUserInteraction);
    bool auth = (result == Authority::Yes);
    //qDebug() << "caller is"<< (( auth )? "" : "not") << "authorized to do:" << args.first();
    if ( auth ) {
        QStringList _args = args;
        if ( !callThread->isRunning() ) {
            callThread->setArgs(_args, 60);
            callThread->start();
        }
        else warningShow("Authoring", "Process is busy.\nTry later.");
    }
    else warningShow("Authoring",
        QString("Not authorized to do %1.").arg(args.first()));
    return auth;
}
void DocContent::resultReceiver(QStringList data)
{
    QStringList _result = data;
    qDebug()<<_result<<"result";
    if ( callThread->isRunning() ) callThread->stop();
    selinuxLed->setEnabled(true);
}
void DocContent::warningShow(QString title, QString msg)
{
    emit warningShowed();
    QString time = QTime::currentTime().toString();
    QString errorMsg = QString("%1 %2:\n%3").arg(time).arg(title).arg(msg);
    appendErrorMsg(errorMsg);
}
void DocContent::appendErrorMsg(QString &msg)
{
    if ( errorLog->toPlainText().count()>10240 ) errorLog->clear();
    errorLog->append(msg);
}
