#include "elemsandbox.h"

ElemSandbox::ElemSandbox(QObject *parent) :
    QObject(parent)
{
  connect(this, SIGNAL(processState(bool)), this, SLOT(setProcessState(bool)));
  //connect(this, SIGNAL(readyRead()), this, SLOT(sendMessage()));
  timerId = 0;
  waitTimerId = 0;
  _diff = 0;

  runAct = new Action(QString("%1.%2").arg(DOMAIN).arg("start"), this);
  runAct->setData("start");
  stopAct = new Action(QString("%1.%2").arg(DOMAIN).arg("stop"), this);
  stopAct->setData("stop");
  connect(runAct, SIGNAL(authorized()), this, SLOT(actionActivated()));
  connect(stopAct, SIGNAL(authorized()), this, SLOT(actionActivated()));
  helperCaller = new HelperCaller(this);
  connect(helperCaller, SIGNAL(resultData(QStringList&)), this, SLOT(resultReceiver(QStringList&)));
  connect(helperCaller, SIGNAL(warning(QString&)), this, SLOT(mainWindowUp()));
}
ElemSandbox::~ElemSandbox()
{
  disconnect(this, SIGNAL(processState(bool)), this, SLOT(setProcessState(bool)));
  //disconnect(this, SIGNAL(readyRead()), this, SLOT(sendMessage()));
  disconnect(runAct, SIGNAL(authorized()), this, SLOT(actionActivated()));
  disconnect(stopAct, SIGNAL(authorized()), this, SLOT(actionActivated()));
  disconnect(helperCaller, SIGNAL(resultData(QStringList&)), this, SLOT(resultReceiver(QStringList&)));
  disconnect(helperCaller, SIGNAL(warning(QString&)), this, SLOT(sendWarning(QString&)));

  delete runAct;
  runAct = 0;
  delete stopAct;
  stopAct = 0;
  delete helperCaller;
  helperCaller = 0;
}
void ElemSandbox::setItemReference(JobItemModel *model, JobItemIndex *idx)
{
  own_model = model;
  own_index = idx;
  name = idx->getName();
  proc_Status.insert("availability", QVariant(AVAILABLE));
  proc_Status.insert("isRunning", QVariant(STOPPED));
  proc_Status.insert("initName", QVariant(name));
  idx->setData(proc_Status);
  settings.beginGroup("Tasks");
  settings.beginGroup(name);
  bool autoRun = settings.value("AutoRun", QVariant()).toBool();
  settings.endGroup();
  settings.endGroup();
  if ( autoRun ) runJob();
}
void ElemSandbox::buildCommand()
{
  settings.beginGroup("Tasks");
  settings.beginGroup(name);
  QString URI = settings.value("URI", QVariant("qemu:///session")).toString();
  bool privileged = settings.value("Privileged", QVariant(false)).toBool();
  bool shell = settings.value("Shell", QVariant(false)).toBool();
  bool verbose = settings.value("Verbose", QVariant(false)).toBool();
  bool debug = settings.value("Debug", QVariant(false)).toBool();
  bool useMount = settings.value("Mount", QVariant(false)).toBool();
  QString mountType = settings.value("MountType", QVariant("host-bind")).toString();
  QString bindRule = settings.value("MountBindRule", QVariant()).toString();
  bool runInTerminal = settings.value("RunInTerminal", QVariant(true)).toBool();
  bool customTerminal = settings.value("CustomTerminal", QVariant(false)).toBool();
  QString Terminal = settings.value("Terminal", QVariant()).toString();
  bool enableIncluded = settings.value("EnableIncluded", QVariant(false)).toBool();
  QString includes = settings.value("Includes", QVariant()).toString();
  bool enableNetwork = settings.value("Network", QVariant(false)).toBool();
  QString netType = settings.value("NetworkType", QVariant("dhcp")).toString();
  QString address = settings.value("Address", QVariant()).toString();
  QString sources = settings.value("Source", QVariant("default")).toString();
  QString macAddr = settings.value("MAC", QVariant()).toString();
  QString route = settings.value("Route", QVariant()).toString();
  bool altSecurity = settings.value("AltSecurity", QVariant(false)).toBool();
  QString securityType = settings.value("AltSecurityType", QVariant("dynamic")).toString();
  QString securitySELabel = settings.value("AltSecSELabel", QVariant()).toString();
  QString kernelForUse = settings.value("KernelForUse", QVariant()).toString();
  QString command = settings.value("Command", QVariant()).toString();
  QString addOption = settings.value("AddOptions", QVariant()).toString();;
  settings.endGroup();
  settings.endGroup();
  commandLine.clear();
  // command building
  if (!command.isEmpty()) {
      // append connectURI
      commandLine.append("-c");
      commandLine.append(URI);
      // append Name
      commandLine.append("-n");
      commandLine.append(name);
      if (useMount) {
          // binding mount rules
          QStringList bindRules = _bindRules(bindRule);
          // append mount options
          if ( !bindRules.isEmpty() ) {
              commandLine.append("-m");
              QString s = bindRules.join(" -m ");
              commandLine.append(s);
          };
      };
      if (enableIncluded) {
          // append includes options
          commandLine.append("-I");
          commandLine.append(includes);
      };
      if (enableNetwork) {
          // append network options
          QString networkOpt;
          if ( netType.toLower()!="dhcp" ) {
              netType.append("=");
              netType.append(address);
          };
          networkOpt.append(netType);
          if ( !sources.isEmpty() ) {
              networkOpt.append(",");
              networkOpt.append("source=");
              networkOpt.append(sources);
          };
          if ( !macAddr.isEmpty() ) {
              networkOpt.append(",");
              networkOpt.append("mac=");
              networkOpt.append(macAddr);
          };
          if ( !route.isEmpty() ) {
              networkOpt.append(",");
              networkOpt.append("route=");
              networkOpt.append(route);
          };
          commandLine.append("-N");
          commandLine.append(networkOpt);
      };
      if (altSecurity) {
          //append security options
          QString s;
          if (securityType=="dynamic" || securityType=="inherit") s = securityType;
          else s = QString("%1=%2").arg(securityType).arg(securitySELabel);
          commandLine.append("-s");
          commandLine.append(s);
      };
      // append kernel options (QEMU only)
      if ( URI.startsWith("qemu:///") && !kernelForUse.isEmpty() ) {
          QString version = kernelForUse.section(KERNEL, -1);
          commandLine.append(QString("--kernver=%1").arg(version));
          commandLine.append(QString("--kernpath=%1").arg(kernelForUse));
      };
      // append additional options
      if (!addOption.isEmpty()) commandLine.append(addOption);
      // append priveleged
      if (privileged) commandLine.append("-p");
      //append shell
      if (shell) commandLine.append("-l");
      // append verbose
      if (verbose) commandLine.append("-v");
      // append debug
      if (debug) commandLine.append("-d");
      // append command
      commandLine.append(command);
      // prepend virt-sandbox command
      commandLine.prepend(QString("virt-sandbox"));
      // prepend terminal if specified
      environment.clear();
      environment << "ENVIRONMENT:";
      environment << QProcess::systemEnvironment();
      if ( !URI.contains("qemu") ) {
          environment.replaceInStrings(QString("HOME=%1").arg(getenv("HOME")), "HOME=/root");
      };
      //environment << QString("PATH=%1").arg( getenv("PATH") );
      if ( runInTerminal ) {
          // set Environment
          if ( !customTerminal ) {
              commandLine.prepend(QString("xdg-terminal-util"));
          }
          else if ( !Terminal.isEmpty() ) {
              QStringList _terminal;
              foreach ( QString s, Terminal.split(" ") ) {
                  if ( !s.isEmpty() ) _terminal.prepend(s);
              };
              foreach ( QString s, _terminal ) {
                  commandLine.prepend(s);
              };
          }
          else {
              commandLine.prepend(QString("-e"));
              commandLine.prepend(QString("xterm"));
          };
      };
  };
}
QStringList ElemSandbox::_bindRules(QString &fileName)
{
    QStringList _list;
    QByteArray _data;
    QFile *f = new QFile(fileName);
    f->open(QIODevice::ReadOnly|QIODevice::Text);
    _data = f->readAll();
    f->close();
    f->deleteLater();
    _list = QTextStream(&_data).readAll().split("\n");
    _list.removeAll("");
    return _list;
}
void ElemSandbox::runJob()
{
  settings.beginGroup("Tasks");
  settings.beginGroup(name);
  checkTimeout = settings.value("TimeOut", QVariant(TIMEOUT)).toInt();
  settings.endGroup();
  settings.endGroup();
  proc_Status.insert("availability", QVariant(NOT_AVAILABLE));
  proc_Status.insert("isRunning", QVariant(RUNNING));
  own_index->setData(proc_Status);
  buildCommand();
  bool result = runAct->activate();
  if ( !result ) {
      addMsgToLog("Authority", "Can't initiate authority.");
      emit processState(STOPPED);
  };
}
void ElemSandbox::killJob()
{
  bool result = stopAct->activate();
  if ( !result ) {
      addMsgToLog("Authority", "Can't initiate authority.");
  };
}
void ElemSandbox::undockJob()
{
    proc_Status.insert("availability", QVariant(NOT_AVAILABLE));
    own_index->setData(proc_Status);
    QModelIndex _idx = own_model->index( own_model->jobItemDataList.indexOf( own_index ), 1 );
    own_model->setData(_idx, "Undocking", Qt::EditRole);
    if (timerId) {
        killTimer(timerId);
        timerId = 0;
    };
    emit processState(STOPPED);
}
void ElemSandbox::setProcessState(bool status)
{
  if ( status ) {
      proc_Status.insert("isRunning", QVariant(RUNNING));
  } else {
      PID.clear();
      PID.append("-");
      proc_Status.insert("isRunning", QVariant(STOPPED));
  };
  proc_Status.insert("availability", QVariant(AVAILABLE));
  own_index->setData(proc_Status);
  int row = own_model->jobItemDataList.indexOf(own_index);
  QString data;
  for (int i=0; i<3; i++) {
      switch (i) {
      case 0:
          data = name;
          break;
      case 1:
          data = (status) ? "Running":"-";
          break;
      case 2:
          data = PID;
          break;
      default:
          break;
      };
      own_model->setData(own_model->index(row, i), data, Qt::EditRole);
  };
}
void ElemSandbox::timerEvent(QTimerEvent *event)
{
    int percent = 0;
    int _timerId = event->timerId();
    //qDebug() << "Timer ID:" << _timerId<< timerId<<"PID:"<<PID.toInt();
    if ( _timerId && timerId==_timerId ) {
        //qDebug() << "Timer ID:" << timerId<<PID<<::kill(PID.toInt(), SIGZERO);
       //if ( ::kill(PID.toInt(), SIGZERO)!=0 ) killJob();
       QStringList args;
       args <<"getProcessState"<<commandLine;
       helperCaller->actionActivated(args);
    } else if ( _timerId && waitTimerId==_timerId ) {
        if ( checkTimeout - _diff + 1 ) {
            percent = int ((float(_diff)/checkTimeout)*100.0);
            QModelIndex _idx = own_model->index( own_model->jobItemDataList.indexOf( own_index ), 1 );
            own_model->setData(_idx, QString::number(percent), Qt::EditRole);
            _diff++;
        } else {
            killTimer(waitTimerId);
            waitTimerId = 0;
            _diff = 0;
            emit processState(RUNNING);
            if (!timerId) timerId = startTimer(1000);
            else addMsgToLog("INFO", QString("Wait Timer is running: %1").arg(waitTimerId));
        };
    };
}
void ElemSandbox::sendMessage()
{
  QByteArray _data;
  //_data = readAllStandardOutput();
  _data.append("\n");
  //_data.append(readAllStandardError());
  qDebug()<<QTextStream(&_data).readAll();
}
void ElemSandbox::actionActivated()
{
  // This slot is called whenever an action is allowed
  // here you will do your dirt job by calling a helper application
  // that might erase your hardrive ;)
  Action *action = qobject_cast<Action *>(sender());

  // this is our Special Action that after allowed will call the helper
  QString method = action->data().toString();
  if ( action->is(QString("%1.%2").arg(DOMAIN).arg(method)) ) {
      //qDebug() << "toggled for: "<< action->actionId();
      if ( commandLine.isEmpty() ) {
          addMsgToLog("INFO", QString("\"%1\" is empty.").arg(name));
          emit processState(STOPPED);
          return;
      };
      QStringList args;
      args << method << commandLine << environment;
      //qDebug()<<args.join(" ");
      bool result = helperCaller->actionActivated(args, checkTimeout);
      //qDebug()<< method<<" authority : "<< result;
      if ( method=="start" ) {
          if ( result ) {
              if (!waitTimerId) waitTimerId = startTimer(1000);
              else
                  addMsgToLog("INFO", QString("Wait Timer is running: %1").arg(waitTimerId));
          }
          else {
              emit processState(STOPPED);
          };
      }
      else if ( method=="stop" ) {
          // do nothing
      };
  };
}
void ElemSandbox::resultReceiver(QStringList &s)
{
    if ( s.isEmpty() ) return;
    QString method = s.at(1);
    if ( method=="getProcessState" ) {
        if ( !QVariant(s.first()).toBool() ) {
            if (timerId)
              {
                killTimer(timerId);
                timerId = 0;
              };
            emit processState(STOPPED);
            addMsgToLog( QString("\"%1\" Job").arg(name),
                         QString("REPLY (action \"%2\"):\n%1").arg(s.last()).arg(method));
        };
    }
    else if ( method=="setCurrentSELinuxMode" ) {
        //qDebug()<<"setCurrentSELinuxMode used for"<< name<<s;
        if ( QVariant(s.first()).toBool() ) {
            // mode is setted
        }
        else {
            emit processState(STOPPED);
            addMsgToLog("INFO", "Set SELinux mode failed.");
        };
    }
    else if ( method=="start" ) {
        if ( QVariant(s.first()).toBool() ) {
            addMsgToLog( QString("\"%1\" Job").arg(name),
                         QString("PID (action \"%2\"): %1").arg(s.last()).arg(method));
            PID.clear();
            PID.append(s.last());
        }
        else {
            killTimer(waitTimerId);
            waitTimerId = 0;
            emit processState(STOPPED);
            addMsgToLog( QString("\"%1\" Job").arg(name),
                         QString("REPLY (action \"%2\"):\n%1").arg(s.last()).arg(method));
        };
    }
    else if ( method=="stop" ) {
        if ( QVariant(s.first()).toBool() ) {
            if (timerId)
              {
                killTimer(timerId);
                timerId = 0;
              };
            emit processState(STOPPED);
        }
        else {
            bool ok;
            int pid = s.last().toInt(&ok);
            if ( !ok || !pid ) {
                if (timerId)
                    {
                        killTimer(timerId);
                        timerId = 0;
                    };
                emit processState(STOPPED);
            }
        };
    };
}
void ElemSandbox::addMsgToLog(QString title, QString msg)
{
    QString time = QTime::currentTime().toString();
    QString errorMsg = QString("<b>%1 %2:</b><br>%3").arg(time).arg(title).arg(msg);
    sendWarning(errorMsg);
    mainWindowUp();
}
void ElemSandbox::sendWarning(QString &msg)
{
    emit warning(msg);
}
void ElemSandbox::mainWindowUp()
{
    emit warningShowed();
}
