#ifndef ELEMSANDBOX_H
#define ELEMSANDBOX_H

#include <QDebug>
#include <QStringList>
#include <QSettings>
#include <QDir>
#include <QTimer>
#include <QTimerEvent>
#include <QCoreApplication>
#include "signal.h"
#include "settings/settings.h"
#include "helper_caller/helper_caller.h"
#include "layout/jobitem_model.h"

#include "polkit-qt-1/polkitqt1-gui-action.h"
using namespace PolkitQt1::Gui;

#define RUNNING true
#define STOPPED false
#define TO_RUN true
#define TO_STOP false
#define SIGZERO 0
#define AVAILABLE true
#define NOT_AVAILABLE false

class ElemSandbox : public QObject
{
    Q_OBJECT

public:
    ElemSandbox(QObject *parent);
    ~ElemSandbox();

public slots:
    void setItemReference(JobItemModel*, JobItemIndex*);
    void runJob();
    void killJob();
    void undockJob();

signals:
    void processState(bool);
    void warningShowed();
    void warning(QString&);

private:
    JobItemModel  *own_model;
    JobItemIndex  *own_index;

    QString        PID;
    QString        name;
    DATA           proc_Status;
    QSettings      settings;
    int            waitTimerId;
    int            timerId;

    QStringList    commandLine;
    int            checkTimeout;
    int            _diff;
    bool           shred;
    bool           mountDirs;

    Action        *runAct;
    Action        *stopAct;
    HelperCaller  *helperCaller;
    QStringList    environment;

private slots:
    void buildCommand();
    QStringList _bindRules(QString&);
    void setProcessState(bool);
    void timerEvent(QTimerEvent*);
    void sendMessage();
    void actionActivated();
    void resultReceiver(QStringList&);
    void addMsgToLog(QString, QString);
    void sendWarning(QString&);
    void mainWindowUp();
};

#endif
