﻿#include "virt_sandbox_run_helper.h"
#include "virtsandboxrunneradaptor.h"

//#include "polkit-qt-1/polkitqt1-authority.h"
#include "selinux/selinux.h"
#include "libvirt/libvirt.h"
#include "libvirt/virterror.h"

#include <QDBusMessage>
#include <QtDBus/QDBusConnection>
#include <QtDBus/QDBusInterface>
#include <QtCore/QTimer>
#include <QtCore/QDebug>
#include <QtXml/QDomDocument>
#include "signal.h"

#define SIGZERO          0
#define MINUTE           60000
#define NOT_AUTH         QStringList()<<QVariant(false).toString()<<"Authority is fail."
#define NOT_STARTED      QStringList()<<QVariant(false).toString()<<"Not started."
#define NOT_OPENED       QStringList()<<QVariant(false).toString()<<"Policy-file is not opened."
#define NOT_FIXED        QStringList()<<QVariant(false).toString()<<"Parameter is not fixed."
#define NOT_AVAILABLE    QStringList()<<QVariant(false).toString()<<"Executive process is not available.\nTry later."
#define NOT_EXIST_STATE  QStringList()<<QVariant(false).toString()<<"Executive process is not exist."
#define NOT_EXIST_STOP   QStringList()<<QVariant(true ).toString()<<"Executive process is not exist."
#define CONNECT_FAILED   QStringList()<<QVariant(false).toString()<<"Connection to the Hypervisor is failed."
#define SHUTDOWN         QStringList()<<QVariant(false).toString()<<"Helper is shutdown.\nTry later."

//using namespace PolkitQt1;

VSRHelper::VSRHelper(int &argc, char **argv)
        : QCoreApplication(argc, argv)
{
    qDebug() << "Creating Helper";
    (void) new VirtsandboxrunnerAdaptor(this);
    // Register the DBus service
    QDBusConnection bus = QDBusConnection::systemBus();
    if (!bus.registerService(SERVICE_DOMAIN)) {
        qDebug() << bus.lastError().message();
        qDebug() << "unable to register the service";
        QTimer::singleShot(0, this, SLOT(quit()));
        return;
    }

    if (!bus.registerObject("/", this)) {
        qDebug() << bus.lastError().message();
        qDebug() << "unable to register path to dbus object";
        QTimer::singleShot(0, this, SLOT(quit()));
        return;
    };
    // Normally you will set a timeout so your application can
    // free some resources of the poor client machine ;)
    QStringList environment;
    environment.append( QString(getenv("PATH")).split(":") );
    environment.removeAll(QString());
    environment.append("/usr/bin");
    environment.append("/usr/sbin");
    environment.append("/usr/local/bin");
    environment.append("/usr/local/sbin");
    environment.append("/bin");
    environment.append("/sbin");
    setenv("PATH", environment.join(":").toLatin1(), 1);
    //qDebug()<< "Helper $PATH=" <<getenv("PATH");
    processes = new QMap<QString, QProcess*>();
    timerId = startTimer(MINUTE);
}

VSRHelper::~VSRHelper()
{
    if (processes) {
        processes->clear();
        delete processes;
        processes = 0;
    };
    qDebug() << "Destroying Helper";
}

/* private slots */
QString VSRHelper::getCurrentSELinuxMode()
{
    QString currentSELinuxMode;
    currentSELinuxMode.append("None");
    int rc;
    rc = is_selinux_enabled();
    if (rc < 0) return currentSELinuxMode;
    currentSELinuxMode.clear();
    if (rc == 1) {
        rc = security_getenforce();
        if (rc < 0) {
            currentSELinuxMode.append("None");
        }
        else if (rc) {
            currentSELinuxMode.append("Enforcing");
        }
        else
            currentSELinuxMode.append("Permissive");
    } else {
        currentSELinuxMode.append("Disabled");
    };
    return currentSELinuxMode;
}
QStringList VSRHelper::selectProcessArgs(const QStringList &args)
{
    QStringList result;
    int separator = args.lastIndexOf("ENVIRONMENT:");
    if ( separator+1 ) {
        foreach (QString var, args) {
            if ( args.indexOf(var) < separator ) {
                result.append(var);
            };
        };
    };
    //qDebug()<<result<<separator<<"ARGS";
    return result;
}
QStringList VSRHelper::selectProcessEnv(const QStringList &args)
{
    QStringList result;
    int separator = args.lastIndexOf("ENVIRONMENT:");
    if ( separator+1 ) {
        foreach (QString var, args) {
            if ( args.indexOf(var) > separator ) {
                result.append(var);
            };
        };
    };
    //qDebug()<<result<<separator<<"ENV";
    return result;
}
QString VSRHelper::getJobName(QStringList &args) const
{
    QString result;
    int i = args.indexOf("-n");
    if ( (i+1) && !args.at(i+1).isEmpty() ) result = args.at(i+1);
    return result;
}
bool VSRHelper::processIsAvailable(QString &jobName) const
{
    return ( processes->value(jobName)->state()==QProcess::NotRunning );
}
void VSRHelper::errorMSG()
{
    QProcess *process = qobject_cast<QProcess*>(sender());
    QByteArray _errors;
    _errors.append( process->readAllStandardOutput() );
    _errors.append("\n");
    _errors.append( process->readAllStandardError() );
    QTextStream stream(&_errors);
    QString errors = stream.readAll();
    qDebug()<<"************"<<processes->key(process)<<"************";
    qDebug()<<errors;
    QStringList msg;
    msg.append(processes->key(process));
    msg.append(errors);
    emit helperMessage(msg);
}
void VSRHelper::timerEvent(QTimerEvent *event)
{
    int _timerId = event->timerId();
    //qDebug() << "Timer ID:" << _timerId<< timerId;
    if ( _timerId && timerId==_timerId ) {
        if ( ! processes->size() ) {
            killTimer( timerId );
            timerId = 0;
            quit();
        }
        else {
            QStringList _toDelete;
            foreach ( QString jobName, processes->keys() ) {
                if ( processIsAvailable(jobName) ) _toDelete.append(jobName);
            };
            foreach ( QString jobName, _toDelete ) {
                disconnect(processes->value(jobName), SIGNAL(readyRead()), this, SLOT(errorMSG()));
                processes->remove(jobName);
                qDebug()<<jobName<<"executive process removed.";
            };
        };
    };
}
QStringList VSRHelper::readChildren(QString &pid)
{
    QList<QString>::const_iterator i;
    QStringList result;
    QStringList _children;
    QString filename;
    filename = QString("/proc/%1/task/%1/children").arg(pid);
    QFile f;
    f.setFileName(filename);
    if (f.exists()) {
        f.open(QIODevice::ReadOnly|QIODevice::Text);
        f.waitForReadyRead(10000);
        f.setTextModeEnabled(true);
        QByteArray _data;
        _data = f.readAll();
        _children.append((QString().append(_data.data())).split(" "));
    };
    f.close();

    for (i = _children.constBegin(); i != _children.constEnd(); ++i) {
        QString child_pid;
        if ( !(*i).isEmpty() ) {
            child_pid = (*i);
            result.append(child_pid);
            result.append(readChildren(child_pid));
        };
    };
    return result;
}
QStringList VSRHelper::findPotentialChildren(QString &name)
{
    QStringList result;
    QFile f;
    QString filename;
    bool isNumber;
    QByteArray _data;
    QString cmdline;
    QDir dir = QDir("/proc");
    QStringList dirs = dir.entryList(QDir::AllDirs);
    foreach (QString item, dirs) {
        item.toULong(&isNumber);
        if ( isNumber ) {
            filename = QString("/proc/%1/cmdline").arg(item);
            f.setFileName(filename);
            if (f.exists()) {
                f.open(QIODevice::ReadOnly);
                f.waitForReadyRead(10000);
                _data = f.readAll();
                QTextStream _cmdline(&_data);
                _cmdline.setAutoDetectUnicode(true);
                while (!_cmdline.atEnd()) {
                    _cmdline.skipWhiteSpace();
                    QChar sign;
                    _cmdline>>sign;
                    cmdline.append( sign );
                };
                //qDebug()<<item<<cmdline;
            };
            f.close();
            if ( cmdline.contains(name) ) {
                //qDebug()<<item<<"matched"<<name<<cmdline;
                result.append(item);
            };
            cmdline.clear();
        };
    }
    return result;
}

/* public slots */
QStringList VSRHelper::start(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString jobName = getJobName(QStringList()<<args);
    if ( processes->contains(jobName) && !processIsAvailable(jobName) ) return NOT_AVAILABLE;
    QDBusMessage msg;
    msg = message();

    //QString method = QString("%1.%2").arg(SERVICE_DOMAIN).arg("start");
    // message().service() is the service name of the caller
    // We can check if the caller is authorized to the following action
    //Authority::Result result;
    //SystemBusNameSubject subject(msg.service());
    //result = Authority::instance()->checkAuthorizationSync(method,
    //         subject , Authority::AllowUserInteraction);
    //if (result == Authority::Yes) {
    if (true) {
        //qDebug() << msg.service() << method;
        //emit helperMessage(QStringList()<< msg.service() << method);
        QString runApp;
        runApp = args.first();
        QStringList Args;
        Args.append( selectProcessArgs(args) );
        Args.removeFirst();
        QProcess *process;
        if ( !processes->contains(jobName) ) {
            process = new QProcess(this);
            processes->insert(jobName, process);
            connect(processes->value(jobName), SIGNAL(readyRead()), this, SLOT(errorMSG()));
        }
        else process = processes->value(jobName);
        // *** set Environment
        QStringList env = QProcess::systemEnvironment();
        env.append( selectProcessEnv(args) );
        //qDebug()<<env<<"Environment";
        process->setEnvironment(env);
        //qDebug()<<process->processEnvironment().toStringList()<<"setted Environment";
        // *** set Environment
        qDebug() << "VSRHelper::start" << jobName;
        process->start(runApp, Args);
        bool started = process->waitForStarted();
        QString PID;
        PID = (started) ? QString::number(process->pid()) : "-1";
        qDebug()<<runApp<<Args.join(" ")<<PID;
        if ( started ) return QStringList()<<QVariant(started).toString()<<PID;
        else return QStringList()<<NOT_STARTED;
    } else {
        qDebug() << msg.service() << QString("Can't set the implicit authorization");
        return NOT_AUTH;
    };
}
QStringList VSRHelper::stop(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString jobName = getJobName(QStringList()<<args);
    if ( !processes->contains(jobName) ) return NOT_EXIST_STOP;
    QDBusMessage msg;
    msg = message();

    //QString method = QString("%1.%2").arg(SERVICE_DOMAIN).arg("stop");
    // message().service() is the service name of the caller
    // We can check if the caller is authorized to the following action
    //Authority::Result result;
    //SystemBusNameSubject subject(msg.service());
    //result = Authority::instance()->checkAuthorizationSync(method,
    //         subject , Authority::AllowUserInteraction);
    //if (result == Authority::Yes) {
    if (true) {
        //emit helperMessage(QStringList()<< msg.service() << method);
        bool state = false;
        if ( !processes->contains(jobName) ) return NOT_EXIST_STATE;
        QProcess *process;
        process = processes->value(jobName);
        if ( process->state()==QProcess::Running ) {
            qDebug() << "VSRHelper::stop"<< jobName;
            /* get process children */
            QString PID = QString::number( process->pid() );
            QStringList children;
            children = readChildren(PID);
            qDebug()<<children<<jobName;
            findPotentialChildren(jobName);
            /* kill process with children */
            process->kill();
            QList<QString>::const_iterator i;
            for (i = children.constBegin(); i != children.constEnd(); ++i)
              {
                pid_t _pid = (*i).toInt();
                if ( ::kill(_pid, SIGZERO)==0 )
                  {
                    //qDebug() <<_pid<< ::kill(_pid, SIGZERO);
                    ::kill(_pid, SIGKILL);
                  };
              };
            /* TODO: kill process Potential children */
        };
        state = ( process->state()==QProcess::NotRunning );
        return QStringList()<<QVariant(state).toString()<<QString::number( process->pid() );
    }
    else {
        qDebug() << msg.service() << QString("Can't set the implicit authorization");
        return NOT_AUTH;
    }
}
QStringList VSRHelper::getProcessState(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString jobName = getJobName(QStringList()<<args);
    //qDebug() << "VSRHelper::getProcessState"<< jobName;
    if ( !processes->contains(jobName) ) return NOT_EXIST_STATE;
    QProcess *process = processes->value(jobName);
    bool state = ( process->state()==QProcess::Running );
    return QStringList()<<QVariant(state).toString()<<QString::number(process->pid());
}
QStringList VSRHelper::setAuthType(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QDBusMessage msg;
    msg = message();

    QString method = QString("%1.%2").arg(SERVICE_DOMAIN).arg("setAuthType");
    //Authority::Result result;
    //SystemBusNameSubject subject(msg.service());
    //result = Authority::instance()->checkAuthorizationSync(method,
    //         subject , Authority::AllowUserInteraction);
    //if (result == Authority::Yes) {
    if (true) {
        QDomDocument doc = QDomDocument("policy");
        QFile file(QString("/usr/share/polkit-1/actions/%1.%2").arg(SERVICE_DOMAIN).arg("policy"));
        if (!file.open(QIODevice::ReadOnly))
            return NOT_OPENED;
        doc.setContent(&file);
        file.close();
        QDomElement el = doc.firstChildElement("policyconfig").
                         firstChildElement("action");
        bool corrected = false;
        while (!el.isNull()) {
            if ( el.attribute("id", QString())==QString("%1.%2").arg(SERVICE_DOMAIN).arg(args.first()) ) {
                QDomElement _el = el.firstChildElement("defaults");
                _el = _el.firstChildElement("allow_active");
                if (!_el.isNull()) {
                    _el.firstChild().toText().setData(args.last());
                    qDebug()<<"corrected";
                    corrected = true;
                    QDomElement msg = el.firstChildElement("message");
                    //QDomElement icon = el.firstChildElement("icon_name");
                    QString _msg;
                    //QString _icon;
                    if ( args.last().contains("admin") ) {
                        _msg.append("Administrator access is required");
                        //_icon.append("security-high");
                    } else if (args.last().contains("self")) {
                        _msg.append("User access is required");
                        //_icon.append("security-medium");
                    } else if (args.last()=="yes") {
                        _msg.append("Access allowed");
                        //_icon.append("security-low");
                    } else if (args.last()=="no") {
                        _msg.append("Access denied");
                        //_icon.append("call-stop");
                    };
                    if (!msg.isNull()) msg.firstChild().toText().setData(_msg);
                    //if (!icon.isNull()) icon.firstChild().toText().setData(_icon);
                };
                break;
            }
            else el = el.nextSiblingElement("action");
        };
        if ( corrected && file.open(QIODevice::WriteOnly) ) {
            QTextStream stream(&file);
            doc.save(stream, 2);
            file.close();
            return QStringList()<<QVariant(corrected).toString()<<args.first()<<args.last();
        }
        else return NOT_FIXED;
    }
    else return NOT_AUTH;
}
QStringList VSRHelper::getAuthType(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QDBusMessage msg;
    msg = message();

    QString method = QString("%1.%2").arg(SERVICE_DOMAIN).arg("getAuthType");
    //Authority::Result result;
    //SystemBusNameSubject subject(msg.service());
    //result = Authority::instance()->checkAuthorizationSync(method,
    //         subject , Authority::AllowUserInteraction);
    //if (result == Authority::Yes) {
    if (true) {
        QDomDocument doc = QDomDocument("policy");
        QFile file(QString("/usr/share/polkit-1/actions/%1.%2").arg(SERVICE_DOMAIN).arg("policy"));
        if (!file.open(QIODevice::ReadOnly))
            return NOT_OPENED;
        doc.setContent(&file);
        file.close();
        QDomElement el = doc.firstChildElement("policyconfig").
                         firstChildElement("action");
        bool found = false;
        QString auth_type;
        while (!el.isNull()) {
            if ( el.attribute("id", QString())==QString("%1.%2").arg(SERVICE_DOMAIN).arg(args.first()) ) {
                QDomElement _el = el.firstChildElement("defaults");
                _el = _el.firstChildElement("allow_active");
                if (!_el.isNull()) {
                    auth_type = _el.firstChild().toText().data();
                    found = true;
                };
                break;
            }
            else el = el.nextSiblingElement("action");
        };
        return QStringList()<<QVariant(found).toString()<<args.first()<<auth_type;
    }
    else return NOT_AUTH;
}
QStringList VSRHelper::setCurrentSELinuxMode(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QDBusMessage msg;
    msg = message();

    QString method = QString("%1.%2").arg(SERVICE_DOMAIN).arg("setCurrentSELinuxMode");
    // message().service() is the service name of the caller
    // We can check if the caller is authorized to the following action
    //Authority::Result result;
    //SystemBusNameSubject subject(msg.service());
    //result = Authority::instance()->checkAuthorizationSync(method,
    //         subject , Authority::AllowUserInteraction);
    //if (result == Authority::Yes) {
    if (true) {
        bool Result;
        int arg = (args.first().toInt()) ? 1 : 0;
        QStringList SEModes;
        QString currentSELinuxMode;
        if ( arg ) {
            SEModes<<"enforcing";
        }
        else {
            SEModes<<"permissive"<<"disabled";
        };
        currentSELinuxMode.clear();
        currentSELinuxMode = getCurrentSELinuxMode();
        if ( !SEModes.contains( currentSELinuxMode ) ) {
            // setEnforce new mode
            int rc = 0;
            if (is_selinux_enabled() <= 0) {
                /* SELinux is disabled */
                Result = (arg) ? false : true;
            }
            else {
                rc = security_setenforce(arg);
                if (rc < 0) {
                    /* setenforce() failed */
                    Result = false;
                }
                else Result = true;
            };
        }
        else Result = true;
        return QStringList()<<QVariant( Result ).toString()<<currentSELinuxMode;
    }
    else  {
        qDebug() << msg.service() << QString("Can't set the implicit authorization");
        return NOT_AUTH;
    };
}
QStringList VSRHelper::getVirtNetList(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QStringList virtNetList;
    virConnectPtr conn = virConnectOpenReadOnly(NULL);
    if (conn==NULL) return CONNECT_FAILED;
    virNetworkPtr *network;
    virErrorPtr error;
    unsigned int flags = VIR_CONNECT_LIST_NETWORKS_ACTIVE |
                         VIR_CONNECT_LIST_NETWORKS_INACTIVE;
    int ret = virConnectListAllNetworks( conn, &network, flags);
    if ( ret<0 ) {
        error = virConnGetLastError(conn);
        free(network);
        virConnectClose(conn);
        virtNetList<<QVariant(false).toString();
        virtNetList<<QString("%1 : %2").arg(error->code).arg(error->message);
        return virtNetList;
    };
    virtNetList<<QVariant(true).toString();
    //qDebug()<<QString(virConnectGetURI(conn));

    int i = 0;
    while ( network[i] != NULL ) {
        QStringList currentAttr;
        QString autostartStr;
        int is_autostart = 0;
        if (virNetworkGetAutostart(network[i], &is_autostart) < 0) {
            autostartStr.append("no autostart");
        }
        else autostartStr.append( is_autostart ? "yes" : "no" );
        currentAttr<< QString( virNetworkGetName(network[i]) )
                   << QString( virNetworkIsActive(network[i]) ? "active" : "inactive" )
                   << autostartStr
                   << QString( virNetworkIsPersistent(network[i]) ? "yes" : "no" );
        virtNetList.append(currentAttr.join(" "));
        //qDebug()<<currentAttr;
        virNetworkFree(network[i]);
        i++;
    };
    free(network);
    virConnectClose(conn);
    return virtNetList;
}
QStringList VSRHelper::createVirtNetFromXML(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString path = args.first();
    QByteArray xmlData;
    QFile f;
    f.setFileName(path);
    if ( !f.open(QIODevice::ReadOnly) ) {
        return QStringList()<<QVariant(false).toString()<<QString("File \"%1\"\nnot opened.").arg(path);
    };
    xmlData = f.readAll();
    f.close();
    virErrorPtr error;
    virConnectPtr conn = virConnectOpen(NULL);
    if (conn==NULL) return CONNECT_FAILED;
    error = virConnGetLastError(conn);
    virNetworkPtr network = virNetworkCreateXML(conn, xmlData.data());
    if ( network==NULL ) {
        error = virConnGetLastError(conn);
        return QStringList()<<QVariant(false).toString()
                            <<QString("Create Network from\n\"%1\"\nis failed.").arg(path)
                            <<QString("%1 : %2").arg(error->code).arg(error->message);
    };
    virNetworkFree(network);
    virConnectClose(conn);
    return QStringList()<<QVariant(true).toString()<<QString("Create Network from\n\"%1\"\nis success.").arg(path);
}
QStringList VSRHelper::defineVirtNetFromXML(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString path = args.first();
    QByteArray xmlData;
    QFile f;
    f.setFileName(path);
    if ( !f.open(QIODevice::ReadOnly) ) {
        return QStringList()<<QVariant(false).toString()<<QString("File \"%1\"\nnot opened.").arg(path);
    };
    xmlData = f.readAll();
    f.close();
    virErrorPtr error;
    virConnectPtr conn = virConnectOpen(NULL);
    if (conn==NULL) return CONNECT_FAILED;
    virNetworkPtr network = virNetworkDefineXML(conn, xmlData.data());
    if ( network==NULL ) {
        error = virConnGetLastError(conn);
        return QStringList()<<QVariant(false).toString()
                            <<QString("Define Network from\n\"%1\"\nis failed.").arg(path)
                            <<QString("%1 : %2").arg(error->code).arg(error->message);
    };
    virNetworkFree(network);
    virConnectClose(conn);
    return QStringList()<<QVariant(true).toString()
                        <<QString("Define Network from\n\"%1\"\nis success.").arg(path);
}
QStringList VSRHelper::destroyVirtNetwork(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString name = args.first();
    virConnectPtr conn = virConnectOpen(NULL);
    if (conn==NULL) return CONNECT_FAILED;
    virNetworkPtr *network;
    virErrorPtr error;
    unsigned int flags = VIR_CONNECT_LIST_NETWORKS_ACTIVE |
                         VIR_CONNECT_LIST_NETWORKS_INACTIVE;
    int ret = virConnectListAllNetworks( conn, &network, flags);
    if ( ret<0 ) {
        error = virConnGetLastError(conn);
        free(network);
        virConnectClose(conn);
        return QStringList()<<QVariant(false).toString()
                            <<QString("Get Network List Error.")
                            <<QString("%1 : %2").arg(error->code).arg(error->message);
    };
    //qDebug()<<QString(virConnectGetURI(conn));

    int i = 0;
    bool deleted = false;
    while ( network[i] != NULL ) {
        if ( !deleted && QString( virNetworkGetName(network[i]) )==name ) {
            deleted = (virNetworkDestroy(network[i])+1) ? true : false;
        };
        //qDebug()<<currentAttr;
        virNetworkFree(network[i]);
        i++;
    };
    if (!deleted) error = virConnGetLastError(conn);
    free(network);
    virConnectClose(conn);
    QStringList answer;
    answer<<QVariant(deleted).toString()
          <<QString("\"%1\" Network %2 Destroyed.").arg(name).arg((deleted)?"":"don't");
    if (!deleted) answer<<QString("%1 : %2").arg(error->code).arg(error->message);
    return answer;
}
QStringList VSRHelper::undefineVirtNetwork(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString name = args.first();
    virConnectPtr conn = virConnectOpen(NULL);
    if (conn==NULL) return CONNECT_FAILED;
    virNetworkPtr *network;
    virErrorPtr error;
    unsigned int flags = VIR_CONNECT_LIST_NETWORKS_ACTIVE |
                         VIR_CONNECT_LIST_NETWORKS_INACTIVE;
    int ret = virConnectListAllNetworks( conn, &network, flags);
    if ( ret<0 ) {
        error = virConnGetLastError(conn);
        free(network);
        virConnectClose(conn);
        return QStringList()<<QVariant(false).toString()
                            <<QString("Get Network List Error.")
                            <<QString("%1 : %2").arg(error->code).arg(error->message);
    };
    //qDebug()<<QString(virConnectGetURI(conn));

    int i = 0;
    bool deleted = false;
    while ( network[i] != NULL ) {
        if ( !deleted && QString( virNetworkGetName(network[i]) )==name ) {
            deleted = (virNetworkUndefine(network[i])+1) ? true : false;
        };
        //qDebug()<<currentAttr;
        virNetworkFree(network[i]);
        i++;
    };
    if (!deleted) error = virConnGetLastError(conn);
    free(network);
    virConnectClose(conn);
    QStringList answer;
    answer<<QVariant(deleted).toString()
          <<QString("\"%1\" Network %2 Undefined.").arg(name).arg((deleted)?"":"don't");
    if (!deleted) answer<<QString("%1 : %2").arg(error->code).arg(error->message);
    return answer;
}
QStringList VSRHelper::startVirtNetwork(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString name = args.first();
    virConnectPtr conn = virConnectOpen(NULL);
    if (conn==NULL) return CONNECT_FAILED;
    virNetworkPtr *network;
    virErrorPtr error;
    unsigned int flags = VIR_CONNECT_LIST_NETWORKS_ACTIVE |
                         VIR_CONNECT_LIST_NETWORKS_INACTIVE;
    int ret = virConnectListAllNetworks( conn, &network, flags);
    if ( ret<0 ) {
        error = virConnGetLastError(conn);
        free(network);
        virConnectClose(conn);
        return QStringList()<<QVariant(false).toString()
                            <<QString("Get Network List Error.")
                            <<QString("%1 : %2").arg(error->code).arg(error->message);
    };
    //qDebug()<<QString(virConnectGetURI(conn));

    int i = 0;
    bool started = false;
    while ( network[i] != NULL ) {
        if ( !started && QString( virNetworkGetName(network[i]) )==name ) {
            started = (virNetworkCreate(network[i])+1) ? true : false;
        };
        //qDebug()<<currentAttr;
        virNetworkFree(network[i]);
        i++;
    };
    if (!started) error = virConnGetLastError(conn);
    free(network);
    virConnectClose(conn);
    QStringList answer;
    answer<<QVariant(started).toString()
          <<QString("\"%1\" Network %2 Started.").arg(name).arg((started)?"":"don't");
    if (!started) answer<<QString("%1 : %2").arg(error->code).arg(error->message);
    return answer;
}
QStringList VSRHelper::setAutostartVirtNetwork(const QStringList &args)
{
    if (!timerId) return SHUTDOWN;
    QString name = args.first();
    int autostart;
    if ( args.count()<2 || args.at(1).isEmpty() ) {
        return QStringList()<<QVariant(false).toString()<<QString("Incorrect parameters.");
    } else {
        bool converted;
        int res = args.at(1).toInt(&converted);
        if (converted) autostart = (res) ? 1 : 0;
        else return QStringList()<<QVariant(false).toString()<<QString("Incorrect parameters.");
    };
    virConnectPtr conn = virConnectOpen(NULL);
    if (conn==NULL) return CONNECT_FAILED;
    virNetworkPtr *network;
    virErrorPtr error;
    unsigned int flags = VIR_CONNECT_LIST_NETWORKS_ACTIVE |
                         VIR_CONNECT_LIST_NETWORKS_INACTIVE;
    int ret = virConnectListAllNetworks( conn, &network, flags);
    if ( ret<0 ) {
        error = virConnGetLastError(conn);
        free(network);
        virConnectClose(conn);
        return QStringList()<<QVariant(false).toString()
                            <<QString("Get Network List Error.")
                            <<QString("%1 : %2").arg(error->code).arg(error->message);
    };
    //qDebug()<<QString(virConnectGetURI(conn));

    int i = 0;
    bool set = false;
    while ( network[i] != NULL ) {
        if ( !set && QString( virNetworkGetName(network[i]) )==name ) {
            set = (virNetworkSetAutostart(network[i], autostart)+1) ? true : false;
        };
        //qDebug()<<currentAttr;
        virNetworkFree(network[i]);
        i++;
    };
    if (!set) error = virConnGetLastError(conn);
    free(network);
    virConnectClose(conn);
    QStringList answer;
    answer<<QVariant(set).toString()
          <<QString("\"%1\" Network %2 Set.").arg(name).arg((set)?"":"don't");
    if (!set) answer<<QString("%1 : %2").arg(error->code).arg(error->message);
    return answer;
}
