#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QSettings>
#include <QCloseEvent>
#include <QMainWindow>
#include <QStringList>
#include <QDockWidget>
#include "doc_content.h"
#include "tray/traywidget.h"
#include "layout/listwidget.h"
#include "toolbar/toolbar.h"
#include "wait_thread.h"
#include "selinux/selinux.h"
#include "virt_network/virtnet_control.h"
#include <QDebug>

#include "polkit-qt-1/polkitqt1-authority.h"
using namespace PolkitQt1;

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

signals:

public slots:

private :
    QSettings        settings;
    JobList         *jobWidget;
    TrayIcon        *trayIcon;
    ToolBar         *toolBar;
    Wait            *wait_thread = NULL;
    QDockWidget     *doc;
    DocContent      *docContent;
    int              timerId;
    QString          initialSELinuxState;
    VirtNetControl  *virtNetControl = NULL;

private slots:
    void closeEvent(QCloseEvent*);
    void closeEvent();
    void changeVisibility();
    void mainWindowUp();
    void initDockWidget();
    void initTrayIcon();
    void trayIconActivated(QSystemTrayIcon::ActivationReason);
    void initJobWidget();
    void initToolBar();
    void editCurrentJobItem();
    void createNewJobItem();
    void deleteCurrentJobItem();
    void removeJobItem(QString&);
    void runCurrentJob();
    void undockCurrentJob();
    void stopCurrentJob();
    void stopJob(int);
    void stopAllJob();
    bool runningJobsExist();
    void mouseMoveEvent(QMouseEvent*);
    void timerEvent(QTimerEvent*);
    void showDoc();
    void selinuxStateChanged(const QString&);
    bool restoreSELinuxState();
    void autoHide();
    void showVirtNetDialog();
    void closeVirtNetDialog();
    void writeToErrorLog(QString&);

};

#endif // MAINWINDOW_H
