#include "common_settings.h"

/* -c -n -l -d -v -p */

CommonSet::CommonSet(QWidget *parent) :
    QWidget(parent)
{
  setContentsMargins(1,1,1,1);
  gridLayout = new QVBoxLayout();
  gridLayout->setAlignment(Qt::AlignRight);
  privileged = new QCheckBox("Privileged inside the sandbox", this);
  privileged->setToolTip("Retain root privileges inside the sandbox,\nrather than dropping privileges to match\nthe current user identity.");
  nameEdit = new QLineEdit(this);
  nameEdit->setPlaceholderText("Enter Job Name");
  autoRun = new QCheckBox("AutoRun", this);
  autoRun->setToolTip("Run Job at startup.");
  shell = new QCheckBox("Launch an interactive shell", this);
  shell->setToolTip("Launch an interactive shell\non a secondary console device");
  verbose = new QCheckBox("Verbose", this);
  verbose->setToolTip("Display verbose progress information");
  debug = new QCheckBox("Debug", this);
  debug->setToolTip("Display debugging information");
  initTypeBox();
  initCmdWidget();
  initTimeoutWidget();
  appendOption = new QLineEdit(this);
  appendOption->setPlaceholderText("Enter additional options");
  gridLayout->addWidget(nameEdit);
  gridLayout->addWidget(autoRun);
  gridLayout->addWidget(privileged);
  gridLayout->addWidget(shell);
  gridLayout->addWidget(verbose);
  gridLayout->addWidget(debug);
  gridLayout->addWidget(typeWdg);
  gridLayout->addWidget(timeoutWdg);
  gridLayout->addWidget(cmdWidget);
  gridLayout->addWidget(appendOption);

  setLayout(gridLayout);
}
CommonSet::~CommonSet()
{
  disconnect(sandboxType, SIGNAL(currentIndexChanged(QString)), this, SLOT(setTypeToolTip(QString)));
  disconnect(selectFile, SIGNAL(clicked()), this, SLOT(setCommandPath()));

  delete autoRun;
  autoRun = 0;
  delete privileged;
  privileged = 0;
  delete shell;
  shell = 0;
  delete verbose;
  verbose = 0;
  delete debug;
  debug = 0;
  delete nameEdit;
  nameEdit = 0;
  delete sandboxType;
  sandboxType = 0;
  delete typeLabel;
  typeLabel = 0;
  delete typeLayout;
  typeLayout = 0;
  delete typeWdg;
  typeWdg = 0;
  delete selectFile;
  selectFile = 0;
  delete command;
  command = 0;
  delete appendOption;
  appendOption = 0;
  delete checkTimeout;
  checkTimeout = 0;
  delete timeoutLabel;
  timeoutLabel = 0;
  delete timeoutLayout;
  timeoutLayout = 0;
  delete timeoutWdg;
  timeoutWdg = 0;
  delete cmdLayout;
  cmdLayout = 0;
  delete cmdWidget;
  cmdWidget = 0;
  delete gridLayout;
  gridLayout = 0;
}
void CommonSet::initTypeBox()
{
  typeWdg = new QWidget(this);
  typeLayout = new QHBoxLayout();
  typeLabel = new QLabel("--connect=URI", this);
  sandboxType = new QComboBox(this);
  sandboxType->setSizeAdjustPolicy(QComboBox::AdjustToContents);
  sandboxType->setToolTip("Set the libvirt connection URI,\ndefaults to qemu:///session if omitted.\nCurrently only the QEMU and LXC drivers are supported.");
  connect(sandboxType, SIGNAL(currentIndexChanged(QString)), this, SLOT(setTypeToolTip(QString)));
  sandboxType->addItem("for QEMU", "qemu:///session");
  sandboxType->addItem("for LXC", "lxc:///");
  typeLayout->addWidget(typeLabel);
  typeLayout->addWidget(sandboxType);
  typeWdg->setLayout(typeLayout);
}
void CommonSet::setTypeToolTip(QString s)
{
  sandboxType->setToolTip(QString("The libvirt connection URI %1:\n< -c %2 >").arg(s).arg(sandboxType->itemData(sandboxType->currentIndex()).toString()));
  //if ( "for LXC"==s ) typeLabel->setText("--connect=URI");
  //else if ( "for QEMU"==s ) typeLabel->setText("--connect=URI\n(need to disable SELinux)");
  //else typeLabel->setText("--connect=URI");
}
void CommonSet::initTimeoutWidget()
{
  timeoutWdg = new QWidget(this);
  timeoutLayout = new QHBoxLayout();
  timeoutLabel = new QLabel("Timeout for wait reply", this);
  checkTimeout = new QSpinBox(this);
  checkTimeout->setToolTip("If the task is not killed application,\nit means that the application\ndoes not have time to identify all child processes.\nTry to increase the timeout for.");
  checkTimeout->setMaximum(10000);
  checkTimeout->setMinimum(1);
  checkTimeout->setSingleStep(1);
  checkTimeout->setValue(TIMEOUT);
  timeoutLayout->addWidget(timeoutLabel);
  timeoutLayout->addWidget(checkTimeout);
  timeoutWdg->setLayout(timeoutLayout);
}
void CommonSet::initCmdWidget()
{
  cmdWidget = new QWidget(this);
  cmdLayout = new QHBoxLayout();
  command = new QLineEdit(this);
  command->setPlaceholderText("Enter command or path");
  selectFile = new QPushButton(QIcon::fromTheme("edit-find"), "", this);
  selectFile->setToolTip("Search File");
  cmdLayout->addWidget(command);
  cmdLayout->addWidget(selectFile);
  cmdWidget->setLayout(cmdLayout);
  connect(selectFile, SIGNAL(clicked()), this, SLOT(setCommandPath()));
}
void CommonSet::setCommandPath()
{
  QString s = QFileDialog::getOpenFileName(this, "Command Path", "~" );
  command->setText(s);
}
QString CommonSet::get_Job_Name() const
{
  return nameEdit->text();
}
void CommonSet::set_Job_Name(QString &s)
{
  nameEdit->setText(s);
}
