#include "include_settings.h"

/* -I HOST-PATH
 * Parses HOST-PATH @includefile whose elements are in the format
 * GUEST-TARGET=ROOT-PATH. If ROOT_PATH is omitted,
 * then it is assumed to be the same as GUEST-TARGET
 */

IncludeSet::IncludeSet(QWidget *parent) :
    QWidget(parent)
{
  setContentsMargins(1,1,1,1);
  commonLayout = new QGridLayout(this);
  commonLayout->setSpacing(1);
  fileName = QString();
  enabled = new QCheckBox("Enable Included Files", this);
  enabled->setChecked(false);
  enabled->setToolTip("Copy all files listed in inputfile\ninto the appropriate temporary sandbox directories");
  fileList = new QListWidget(this);
  fileList->setEnabled(false);
  fileList->setSelectionMode(QAbstractItemView::MultiSelection);
  editDST = new QLineEdit(this);
  editDST->setPlaceholderText("Enter Destination path");
  editDST->hide();
  setDST = new QPushButton(QIcon::fromTheme("archive-insert-directory"),"", this);
  setDST->setToolTip("Set Destination path");
  setDST->hide();
  addFile = new QPushButton(QIcon::fromTheme("archive-insert"),"", this);
  addFile->setEnabled(false);
  addFile->setToolTip("Add file(s) to list");
  addDir  = new QPushButton(QIcon::fromTheme("archive-insert-directory"),"", this);
  addDir->setEnabled(false);
  addDir->setToolTip("Add directory to list");
  delPath = new QPushButton(QIcon::fromTheme("archive-remove"),"", this);
  delPath->setEnabled(false);
  delPath->setToolTip("Delete path from list");

  connect(fileList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(editItem(QListWidgetItem*)));
  connect(editDST, SIGNAL(returnPressed()), this, SLOT(setSourcePath()));
  connect(setDST, SIGNAL(clicked()), this, SLOT(setSourcePath()));
  connect(enabled, SIGNAL(toggled(bool)), this, SLOT(enableFileList(bool)));
  connect(addFile, SIGNAL(clicked()), this, SLOT(addFilesToList()));
  connect(addDir, SIGNAL(clicked()), this, SLOT(addDirToList()));
  connect(delPath, SIGNAL(clicked()), this, SLOT(delPathFromList()));

  commonLayout->addWidget(enabled, 0, 0, 1, 4);
  commonLayout->addWidget(fileList, 2, 0, 5, 4);
  commonLayout->addWidget(editDST, 7, 0);
  commonLayout->addWidget(setDST, 7, 3);
  commonLayout->addWidget(addFile, 12, 0, Qt::AlignLeft);
  commonLayout->addWidget(addDir, 12, 1, Qt::AlignLeft);
  commonLayout->addWidget(delPath, 12, 3, Qt::AlignRight);

  setLayout(commonLayout);
}
IncludeSet::~IncludeSet()
{
  disconnect(fileList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(editItem(QListWidgetItem*)));
  disconnect(editDST, SIGNAL(returnPressed()), this, SLOT(setSourcePath()));
  disconnect(setDST, SIGNAL(clicked()), this, SLOT(setSourcePath()));
  disconnect(enabled, SIGNAL(toggled(bool)), this, SLOT(enableFileList(bool)));
  disconnect(addFile, SIGNAL(clicked()), this, SLOT(addFilesToList()));
  disconnect(addDir, SIGNAL(clicked()), this, SLOT(addDirToList()));
  disconnect(delPath, SIGNAL(clicked()), this, SLOT(delPathFromList()));

  delete editDST;
  editDST = 0;
  delete setDST;
  setDST = 0;
  delete addFile;
  addFile = 0;
  delete addDir;
  addDir = 0;
  delete delPath;
  delPath = 0;
  delete fileList;
  fileList = 0;
  delete enabled;
  enabled = 0;
  delete commonLayout;
  commonLayout = 0;
}
void IncludeSet::set_FileName(const QString s)
{
  fileName = s;
}
void IncludeSet::enableFileList(bool b)
{
  fileList->setEnabled(b);
  addFile->setEnabled(b);
  addDir->setEnabled(b);
  delPath->setEnabled(b);
}
void IncludeSet::addFilesToList()
{
  show_Widgets();
  differentiator = true;
}
void IncludeSet::addDirToList()
{
  show_Widgets();
  differentiator = false;
}
void IncludeSet::setSourcePath()
{
    QStringList path;
    QStringList Dirs;
    QString dir;
    QString dest;
    if ( differentiator ) {
        path = QFileDialog::getOpenFileNames(this, "Include files", "~");
    } else {
        dir = QFileDialog::getExistingDirectory(this, "Include directory", "~");
        path = get_FilesFromDir(dir);
    };
    path.removeAll(QString());
    QString dst = editDST->text();
    if ( !path.isEmpty() ) {
        QChar sep = QDir::separator();
        Dirs.append(dir.split(sep));
        Dirs.removeAll(QString());
        if ( !Dirs.last().isEmpty() ) {
            if ( !dst.endsWith(sep) ) dst.append(sep);
            dst.append(Dirs.last());
        };
        if ( dir.endsWith(sep) ) dir = dir.remove(dir.size()-1, 1);
        // ╔════      build DST=SRC string      ════╗
        if ( differentiator ) {
            if ( !path.last().isEmpty() ) {
                QFileInfo host_dir;
                host_dir.setFile(path.last());
                dest.append(QString("%1=%2").arg(dst).arg(host_dir.dir().canonicalPath()));
            }
        } else {
            if ( !dir.isEmpty() ) {
                dest.append(QString("%1=%2").arg(dst).arg(dir));
            }
        };
        // ╚════    end build DST=SRC string    ════╝
        if ( !dir.endsWith(sep) ) dir.append(sep);
        if ( !dst.endsWith(sep) ) dst.append(sep);
        foreach (QString item, path) {
            QString _file;
            if ( differentiator ) {
                QFileInfo file;
                file.setFile(item);
                _file = file.fileName();
            } else {
                _file = item.split(dir).last();
            };
            item.prepend("=");
            item.prepend(_file);
            item.prepend(dst);
            fileList->addItem(item);
        };
    } else {
        fileList->addItem(dst);
        if ( !dst.isEmpty() ) dest.append(QString("%1=%2").arg(dst).arg(dst));
    };
    if ( !dest.isEmpty() ) emit destinationAdded(dest);
    removeDuplicates();
    show_Widgets();
}
QStringList IncludeSet::get_FilesFromDir(QString &dir)
{
    QStringList list;
    QChar sep = QDir::separator();
    if ( !dir.isEmpty() )
      {
        if ( !dir.endsWith(sep) ) dir.append(sep);
        QStringList files;
        QStringList dirs;
        QDir Dir = QDir(dir);
        files = Dir.entryList(QStringList()<<"*", QDir::NoDotAndDotDot|QDir::Hidden|QDir::Files);
        foreach (QString path, files) {
            path.prepend(dir);
            list.append(path);
        };
        dirs = Dir.entryList(QStringList()<<"*", QDir::NoDotAndDotDot|QDir::Hidden|QDir::Dirs);
        foreach (QString path, dirs) {
            path.prepend(dir);
            list.append(get_FilesFromDir(path));
        };
      };
    return list;
}
void IncludeSet::delPathFromList()
{
  QList<QListWidgetItem *>::const_iterator i;
  QList<QListWidgetItem *> selected;
  selected = fileList->selectedItems();
  for (i = selected.begin(); i!=selected.end(); i++) { delItem(*i); };
  selected.clear();
}
void IncludeSet::delItem(QListWidgetItem *_item)
{
  if (_item)
    {
      fileList->takeItem(fileList->row(_item));
      //qDebug()<<"delete"<<_file;
    }
  else QMessageBox::information(this, "Info", "Item not exist.");
}
void IncludeSet::removeDuplicates()
{
  QStringList _items = get_Included_Path();
  _items.removeAll(QString());
  _items.removeDuplicates();
  fileList->clear();
  fileList->addItems(_items);
  _items.clear();
}
QStringList IncludeSet::get_Included_Path() const
{
  QStringList list;
  for (int i = 0; i!=fileList->count(); i++) {
      list.append(fileList->item(i)->text());
  };
  return list;
}
QString IncludeSet::get_FileName() const
{
  if ( fileName.isEmpty() ) return QString();
  const QString res;
  QString s = get_Included_Path().join("\n");
  //qDebug()<<fileName<<s<<"Incl::get_FileName()";
  int l;
  QFile *f = new QFile(fileName);
  f->open(QIODevice::WriteOnly|QIODevice::Text);
  l = f->write(s.toUtf8());
  f->close();
  f->deleteLater();
  return ( l == s.toUtf8().length() ) ? fileName : QString();
}
void IncludeSet::setIncludesList(QString s)
{
  if ( s.isEmpty() ) return;
  fileName = s;
  QFile *f = new QFile(fileName);
  if (f->exists())
    {
      QStringList _list;
      QByteArray _data;
      f->open(QIODevice::ReadOnly|QIODevice::Text);
      _data = f->readAll();
      f->close();
      _list = QTextStream(&_data).readAll().split("\n");
      _list.removeAll("");
      fileList->clear();
      fileList->addItems(_list);
    }
  else QMessageBox::information(this, "Info", "ListFile not exist.");
  f->deleteLater();
}
void IncludeSet::show_Widgets()
{
    if ( editDST->isVisible() ) {
        editDST->hide();
        editDST->clear();
        setDST->hide();
        addFile->show();
        addDir->show();
        delPath->show();
        fileList->setFocus();
        if (enabled->isChecked()) fileList->setEnabled(true);
    }
    else {
        editDST->show();
        editDST->clear();
        setDST->show();
        setDST->setFocus();
        addFile->hide();
        addDir->hide();
        delPath->hide();
        fileList->setEnabled(false);
    }
}
void IncludeSet::editItem(QListWidgetItem *item)
{
    if ( !item->flags().testFlag(Qt::ItemIsEditable) ) {
            item->setFlags(item->flags()|Qt::ItemIsEditable);
    };
    fileList->editItem(item);
}
