#include "kernel_set.h"

KernelSet::KernelSet(QWidget *parent) :
    QWidget(parent)
{
    kernels = new QLabel("Kernels:", this);
    kernelForUse = new QLabel("Kernel for Use:", this);
    currentKernel = new QLineEdit(this);
    kernelList = new QComboBox(this);
    kernelList->setDuplicatesEnabled(false);
    kernelPath = new QPushButton(QIcon::fromTheme("nuclear"), "Find Custom Kernel", this);
    commonLayout = new QVBoxLayout();
    commonLayout->addWidget(kernels);
    commonLayout->addWidget(kernelList);
    commonLayout->addWidget(kernelPath);
    commonLayout->addWidget(kernelForUse);
    commonLayout->addWidget(currentKernel);
    commonLayout->insertStretch(-1);
    setLayout(commonLayout);
    setList_Of_AvailableStandardKernels();
    connect(kernelList, SIGNAL(currentIndexChanged(QString)), this, SLOT(setKernelForUse(QString)));
    connect(kernelPath, SIGNAL(clicked()), this, SLOT(setCustomKernelForUse()));
}
KernelSet::~KernelSet()
{
    disconnect(kernelList, SIGNAL(currentIndexChanged(QString)), this, SLOT(setKernelForUse(QString)));
    disconnect(kernelPath, SIGNAL(clicked()), this, SLOT(setCustomKernelForUse()));
    delete kernels;
    kernels = 0;
    delete kernelForUse;
    kernelForUse = 0;
    delete currentKernel;
    currentKernel = 0;
    delete kernelList;
    kernelList = 0;
    delete kernelPath;
    kernelPath = 0;
    delete commonLayout;
    commonLayout = 0;
}
void KernelSet::setList_Of_AvailableStandardKernels()
{
    QDir dir = QDir("/boot");
    QStringList allAvailableStandardKernels;
    QString nameFilter = QString("%1%2").arg(KERNEL).arg("*");
    allAvailableStandardKernels.append(dir.entryList(QStringList()<<nameFilter, QDir::Files));
    kernelList->clear();
    kernelList->addItems(allAvailableStandardKernels);
    kernelList->insertSeparator(kernelList->count());
}
void KernelSet::setCustomKernelForUse()
{
    QString nameFilter = QString("Kernels (%1%2)").arg(KERNEL).arg("*");
    QString path;
    path = QFileDialog::getOpenFileName(this, "Find Kernel", "/boot", nameFilter);
    int i = kernelList->findText( path );
    if (i<0) {
        i = kernelList->count();
        kernelList->insertItem(i, path);
        kernelList->setCurrentIndex(i);
    };
}
void KernelSet::setKernelForUse(const QString k)
{
    QString _kernelForUse;
    QString kernelRelease;
    if ( k.isEmpty() ) {
        struct utsname UTSNAME;
        if ( !uname(&UTSNAME) ) {
            kernelRelease.append(UTSNAME.release);
        }
        else _kernelForUse.append(QString("%1%2").arg(KERNEL).arg(kernelRelease));
    }
    else _kernelForUse.append(k);
    if ( k.isEmpty() && kernelRelease.isEmpty() ) {
        currentKernel->clear();
    }
    else if ( _kernelForUse.startsWith(KERNEL) ) {
        currentKernel->setText(QString("/boot/%1").arg(_kernelForUse));
    }
    else currentKernel->setText(_kernelForUse);
    //qDebug()<<k<<_kernelForUse;
}
QString KernelSet::getKernelForUse() const
{
    return currentKernel->text();
}
