#include "network_settings.h"

/* -N NETWORK-OPTIONS */

#define TITLE QString("<b>DESCRIPTION:</b>")
#define COMMENT1 QString("Configure the network interface using DHCP.<br>This key takes no value. No other keys<br>may be specified.")
#define COMMENT2 QString("Configure the network interface with<br>the static IPv4 or IPv6 address IP-ADDRESS.<br>The PREFIX value is the length<br>of the network prefix in IP-ADDRESS.<br>The optional BROADCAST<br>parameter specifies the broadcast address.")
#define SOURCE QString("Set the name of the network to connect the interface to.<br>\"NETWORK\" is the name of any libvirt virtual network.<br>See also virsh net-list.<br>( default, lan, etc. )")
#define MAC QString("Set the MAC address of the network interface,<br>where each NN is a pair of hex digits:<br>NN:NN:NN:NN:NN:NN")
#define ROUTE QString("Configure the network interface with the static<br>IPv4 or IPv6 route IP-NETWORK. The PREFIX value<br>is the length of the network prefix in IP-NETWORK.<br>The optional GATEWAY parameter specifies<br>the address of the gateway.")

#define DHCP QString("dhcp")
#define ADDRESS QString("address")
NetworkSet::NetworkSet(QWidget *parent) :
    QWidget(parent)
{
    setContentsMargins(1, 1, 1, 1);
    commonLayout = new QVBoxLayout();
    useNetwork = new QCheckBox("Add a network interface to the sandbox", this);
    useNetwork->setChecked(false);
    virtControlButton = new QPushButton(this);
    virtControlButton->setIcon(QIcon::fromTheme("preferences-system-network"));
    virtControlButton->setEnabled(false);
    virtControlButton->setToolTip("Show VirtNet Control");
    connect(virtControlButton, SIGNAL(clicked()), this, SLOT(showVirtControl()));
    useNetLayout = new QHBoxLayout();
    useNetLayout->addWidget(useNetwork);
    useNetLayout->addWidget(virtControlButton);
    type = new QComboBox(this);
    type->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    type->setEnabled(false);
    comment = new QLabel(this);
    comment->setEnabled(false);
    address = new QLineEdit(this);
    address->setEnabled(false);
    address->setVisible(false);

    specsLayout = new QHBoxLayout();
    source = new QLineEdit(this);
    source->setVisible(true);
    source->setPlaceholderText("Enter source");
    source->setToolTip(SOURCE);
    source->setEnabled(false);
    mac_address = new QLineEdit(this);
    mac_address->setVisible(true);
    mac_address->setPlaceholderText("Enter MAC-address");
    mac_address->setToolTip(MAC);
    mac_address->setEnabled(false);
    routeTitle = new QLabel(ROUTE, this);
    routeTitle->setVisible(true);
    routeTitle->setEnabled(false);
    route = new QLineEdit(this);
    route->setVisible(true);
    route->setPlaceholderText("IP-NETWORK/PREFIX%GATEWAY");
    route->setToolTip("Example: 192.168.122.255/24%192.168.1.1");
    route->setEnabled(false);
    specsLayout->addWidget(source);
    specsLayout->addWidget(mac_address);

    commonLayout->addItem(useNetLayout);
    commonLayout->addWidget(type);
    commonLayout->addWidget(comment);
    commonLayout->addWidget(address);
    commonLayout->insertStretch(-1);
    commonLayout->addLayout(specsLayout);
    commonLayout->addWidget(routeTitle);
    commonLayout->addWidget(route);
    setLayout(commonLayout);
    QStringList types;
    types<<DHCP<<ADDRESS;
    foreach (QString var, types) {
        type->addItem(var);
    };
    changeComment(types.first());
    connect(useNetwork, SIGNAL(toggled(bool)), this, SLOT(changeNetworkState(bool)));
    connect(type, SIGNAL(currentIndexChanged(QString)), this, SLOT(changeComment(QString)));
    connect(address, SIGNAL(textChanged(const QString&)), this, SLOT(changeAddressLabel(const QString&)));
}
NetworkSet::~NetworkSet()
{
    disconnect(useNetwork, SIGNAL(toggled(bool)), this, SLOT(changeNetworkState(bool)));
    disconnect(virtControlButton, SIGNAL(clicked()), this, SLOT(showVirtControl()));
    disconnect(type, SIGNAL(currentIndexChanged(QString)), this, SLOT(changeComment(QString)));
    disconnect(address, SIGNAL(textChanged(const QString&)), this, SLOT(changeAddressLabel(const QString&)));

    delete useNetwork;
    useNetwork = 0;
    delete virtControlButton;
    virtControlButton = 0;
    commonLayout->removeItem(useNetLayout);
    delete useNetLayout;
    useNetLayout = 0;
    delete type;
    type = 0;
    delete address;
    address = 0;
    delete comment;
    comment = 0;
    delete source;
    source = 0;
    delete mac_address;
    mac_address = 0;
    delete specsLayout;
    specsLayout = 0;
    delete routeTitle;
    routeTitle = 0;
    delete route;
    route = 0;
    delete commonLayout;
    commonLayout = 0;

    if ( virtNetControl!=NULL ) {
      delete virtNetControl;
      virtNetControl = 0;
    };
}
void NetworkSet::changeComment(QString s)
{
    if (s.isEmpty()) return;
    if (s.toLower()==DHCP) {
        comment->setText(QString("%1<br>%2").arg(TITLE).arg(COMMENT1));
        address->setVisible(false);
    }
    else if (s.toLower()==ADDRESS) {
        comment->setText(QString("%1<br>%2").arg(TITLE).arg(COMMENT2));
        address->setVisible(true);
        address->setPlaceholderText("IP-ADDRESS/PREFIX%BROADCAST");
        address->setToolTip("Example:<br>192.168.122.1/24<br>192.168.122.1/24%192.168.122.255<br>2001:212::204:2/64");
    };
    address->setText(type->itemData(type->currentIndex()).toString());
}
void NetworkSet::setCurrentType(QString s)
{
    int i = type->findText(s);
    if (i+1) type->setCurrentIndex(i);
}
QString NetworkSet::getCurrentType() const
{
    return type->currentText();
}
void NetworkSet::changeNetworkState(bool b)
{
    virtControlButton->setEnabled(b);
    type->setEnabled(b);
    comment->setEnabled(b);
    address->setEnabled(b);
    source->setEnabled(b);
    mac_address->setEnabled(b);
    routeTitle->setEnabled(b);
    route->setEnabled(b);
}
void NetworkSet::setCurrentAddress(QString s)
{
    if (s.isEmpty()) return;
    address->setText(s);
}
QString NetworkSet::getCurrentAddress() const
{
    return address->text();
}
void NetworkSet::changeAddressLabel(const QString &s)
{
    if (s.isNull()) return;
    type->setItemData(type->currentIndex(), QVariant(s));
}
QString NetworkSet::getCurrentSource() const
{
    return source->text();
}
void NetworkSet::setCurrentSource(QString s)
{
    if (s.isEmpty()) return;
    source->setText(s);
}
QString NetworkSet::getCurrentMAC() const
{
    return mac_address->text();
}
void NetworkSet::setCurrentMAC(QString s)
{
    if (s.isEmpty()) return;
    mac_address->setText(s);
}
QString NetworkSet::getCurrentRoute() const
{
    return route->text();
}
void NetworkSet::setCurrentRoute(QString s)
{
    if (s.isEmpty()) return;
    route->setText(s);
}
void NetworkSet::showVirtControl()
{
    emit showVirtNetControl();
    Authority::Result result;
    UnixProcessSubject subject(static_cast<uint>(QCoreApplication::applicationPid()));

    result = Authority::instance()->checkAuthorizationSync(
             QString("%1.%2").arg(DOMAIN).arg("getNetworkControlAccess"), subject,
             Authority::AllowUserInteraction);
    bool auth = (result == Authority::Yes);
    if (auth) {
        if ( virtNetControl==NULL ) virtNetControl = new VirtNetControl(this);
        if ( virtNetControl->isHidden() ) {
            virtNetControl->show();
            virtNetControl->startUpdateTimer();
        };
        //virtNetControl->move(mapToGlobal(this->pos()));
    } else QMessageBox::information(this, "Access", "Access denied");
}
