#include "security_settings.h"

/* -s SECURITY-OPTIONS */

#define TITLE QString("<b>DESCRIPTION:</b>")
#define COMMENT1 QString("Dynamically allocate an SELinux label,<br>using the default base context.<br>The default base context is<br>system_u:system_r:svirt_lxc_net_t:s0 for LXC,<br>system_u:system_r:svirt_t:s0 for KVM,<br>system_u:system_r:svirt_tcg_t:s0 for QEMU.")
#define COMMENT2 QString("Dynamically allocate an SELinux label,<br>using the base context USER:ROLE:TYPE:LEVEL,<br>instead of the default base context.")
#define COMMENT3 QString("To set a completely static label. For example,<br>static,label=system_u:system_r:svirt_t:s0:c412,c355")
#define COMMENT4 QString("Inherit the context from the process<br>that is executing virt-sandbox.")

#define DYNAMIC QString("dynamic")
#define DYNAMIC_LABEL QString("dynamic,label")
#define STATIC_LABEL QString("static,label")
#define INHERIT QString("inherit")

SecuritySet::SecuritySet(QWidget *parent) :
    QWidget(parent)
{
    setContentsMargins(1, 1, 1, 1);
    commonLayout = new QVBoxLayout();
    altSecurity = new QCheckBox("Use alternative security options", this);
    altSecurity->setChecked(false);
    type = new QComboBox(this);
    type->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    type->setEnabled(false);
    comment = new QLabel(this);
    comment->setEnabled(false);
    titleLabel = new QLabel("<b>SELinux Label:</b>", this);
    titleLabel->setEnabled(false);
    titleLabel->setVisible(false);
    label = new QLineEdit(this);
    label->setEnabled(false);
    label->setVisible(false);

    commonLayout->addWidget(altSecurity, Qt::AlignLeft);
    commonLayout->addWidget(type);
    commonLayout->addWidget(comment);
    commonLayout->addWidget(titleLabel);
    commonLayout->addWidget(label);
    commonLayout->insertStretch(-1);
    setLayout(commonLayout);
    QStringList types;
    types<<DYNAMIC<<DYNAMIC_LABEL<<STATIC_LABEL<<INHERIT;
    foreach (QString var, types) {
        type->addItem(var);
    };
    changeComment(types.first());
    connect(altSecurity, SIGNAL(toggled(bool)), this, SLOT(changeSecurityState(bool)));
    connect(type, SIGNAL(currentIndexChanged(QString)), this, SLOT(changeComment(QString)));
    connect(label, SIGNAL(textChanged(const QString&)), this, SLOT(changeSELabel(const QString&)));
}
SecuritySet::~SecuritySet()
{
    disconnect(altSecurity, SIGNAL(toggled(bool)), this, SLOT(changeSecurityState(bool)));
    disconnect(type, SIGNAL(currentIndexChanged(QString)), this, SLOT(changeComment(QString)));
    disconnect(label, SIGNAL(textChanged(const QString&)), this, SLOT(changeSELabel(const QString&)));
    delete altSecurity;
    altSecurity = 0;
    delete type;
    type = 0;
    delete label;
    label = 0;
    delete comment;
    comment = 0;
    delete titleLabel;
    titleLabel = 0;
    delete commonLayout;
    commonLayout = 0;
}
void SecuritySet::changeComment(QString s)
{
    if (s.isEmpty()) return;
    if (s.toLower()==DYNAMIC) {
        comment->setText(QString("%1<br>%2").arg(TITLE).arg(COMMENT1));
        titleLabel->setVisible(false);
        label->setVisible(false);
    }
    else if (s.toLower()==DYNAMIC_LABEL) {
        comment->setText(QString("%1<br>%2").arg(TITLE).arg(COMMENT2));
        titleLabel->setVisible(true);
        label->setVisible(true);
        label->setPlaceholderText("system_u:system_r:svirt_t:s0");
    }
    else if (s.toLower()==STATIC_LABEL) {
        comment->setText(QString("%1<br>%2").arg(TITLE).arg(COMMENT3));
        titleLabel->setVisible(true);
        label->setVisible(true);
        label->setPlaceholderText("system_u:system_r:svirt_t:s0:c412,c355");
    }
    else if (s.toLower()==INHERIT) {
        comment->setText(QString("%1<br>%2").arg(TITLE).arg(COMMENT4));
        titleLabel->setVisible(false);
        label->setVisible(false);
    };
    label->setText(type->itemData(type->currentIndex()).toString());
}
void SecuritySet::setCurrentType(QString s)
{
    int i = type->findText(s);
    if (i+1) type->setCurrentIndex(i);
}
QString SecuritySet::getCurrentType() const
{
    return type->currentText();
}
void SecuritySet::changeSecurityState(bool b)
{
    type->setEnabled(b);
    comment->setEnabled(b);
    titleLabel->setEnabled(b);
    label->setEnabled(b);
}
void SecuritySet::setCurrentLabel(QString s)
{
    if (s.isEmpty()) return;
    label->setText(s);
}
QString SecuritySet::getCurrentLabel() const
{
    return label->text();
}
void SecuritySet::changeSELabel(const QString &s)
{
    if (s.isNull()) return;
    type->setItemData(type->currentIndex(), QVariant(s));
}
