#include "settings/settings.h"

SettingsDialog::SettingsDialog(QWidget *parent) :
    QDialog(parent)
{
  setWindowTitle("<noname>");
  restoreGeometry(settings.value("SetDlgGeometry").toByteArray());
  setModal(false);
  setContentsMargins(1,1,1,1);
  initTabWidget();
  warning = new QLabel(this);
  warning->setOpenExternalLinks(true);
  initButtons();
  initParameters();
  fullCommandWdg = new QTextEdit(this);
  fullCommandWdg->setReadOnly(true);
  commonLayout = new QVBoxLayout(this);
  commonLayout->addWidget(tabWidget);
  commonLayout->addWidget(warning);
  commonLayout->addWidget(fullCommandWdg);
  commonLayout->addWidget(buttons);
  commonLayout->insertStretch(-1);
  setLayout(commonLayout);
  timerId = startTimer(1000);
}
SettingsDialog::~SettingsDialog()
{
  disconnect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));
  disconnect(w1->nameEdit, SIGNAL(textChanged(QString)), this, SLOT(set_Title_Name(QString)));
  disconnect(w1->sandboxType, SIGNAL(currentIndexChanged(QString)), this, SLOT(set_Warning_Data(QString)));
  disconnect(w3, SIGNAL(destinationAdded(QString&)), w4, SLOT(add_HOST_BIND_DST(QString&)));
  disconnect(w5, SIGNAL(showVirtNetControl()), this, SLOT(showVirtControl()));
  disconnect(ok, SIGNAL(clicked()), this, SLOT(saveJob()));
  disconnect(cancel, SIGNAL(clicked()), this, SLOT(cancelJob()));

  delete ok;
  ok = 0;
  delete cancel;
  cancel = 0;
  delete w1;
  w1 = 0;
  delete w3;
  w3 = 0;
  delete w4;
  w4 = 0;
  delete w5;
  w5 = 0;
  delete w6;
  w6 = 0;
  delete w7;
  w7 = 0;
  delete tabWidget;
  tabWidget = 0;
  delete warning;
  warning = 0;
  delete fullCommandWdg;
  fullCommandWdg = 0;
  delete buttonsLayout;
  buttonsLayout = 0;
  delete buttons;
  buttons = 0;
  delete commonLayout;
  commonLayout = 0;
}
void SettingsDialog::initTabWidget()
{
  tabWidget = new QTabWidget(this);
  tabWidget->setUsesScrollButtons(true);
  tabWidget->setContentsMargins(1,1,1,1);
  tabWidget->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
  w1 = new CommonSet(this);
  QIcon i1 = QIcon::fromTheme("preferences-other");
  QString n1 = QString("Common");
  w2 = new TerminalSet(this);
  QIcon i2 = QIcon::fromTheme("utilities-terminal");
  QString n2 = QString("Terminal");
  w3 = new IncludeSet(this);
  QIcon i3 = QIcon::fromTheme("document-export");
  QString n3 = QString("Include files");
  w4 = new MountSet(this);
  QIcon i4 = QIcon::fromTheme("folder-sync");
  QString n4 = QString("Work directories");
  w5 = new NetworkSet(this);
  QIcon i5 = QIcon::fromTheme("preferences-system-network");
  QString n5 = QString("Network");
  w6 = new SecuritySet(this);
  QIcon i6 = QIcon::fromTheme("security-high");
  QString n6 = QString("Security");
  w7 = new KernelSet(this);
  QIcon i7 = QIcon::fromTheme("nuclear");
  QString n7 = QString("Kernel");
  tabWidget->addTab(w1, i1, n1);
  tabWidget->addTab(w2, i2, n2);
  tabWidget->addTab(w3, i3, n3);
  tabWidget->addTab(w4, i4, n4);
  tabWidget->addTab(w5, i5, n5);
  tabWidget->addTab(w6, i6, n6);
  tabWidget->addTab(w7, i7, n7);
  connect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));
  connect(w1->nameEdit, SIGNAL(textChanged(QString)), this, SLOT(set_Title_Name(QString)));
  connect(w1->sandboxType, SIGNAL(currentIndexChanged(QString)), this, SLOT(set_Warning_Data(QString)));
  connect(w3, SIGNAL(destinationAdded(QString&)), w4, SLOT(add_HOST_BIND_DST(QString&)));
  connect(w5, SIGNAL(showVirtNetControl()), this, SLOT(showVirtControl()));
}
void SettingsDialog::initButtons()
{
  ok = new QPushButton(QIcon::fromTheme("dialog-ok"),"Ok");
  cancel = new QPushButton(QIcon::fromTheme("dialog-cancel"),"Cancel");
  buttonsLayout = new QHBoxLayout(this);
  buttonsLayout->addWidget(ok);
  buttonsLayout->addWidget(cancel);
  buttons = new QWidget(this);
  buttons->setLayout(buttonsLayout);
  connect(ok, SIGNAL(clicked()), this, SLOT(saveJob()));
  connect(cancel, SIGNAL(clicked()), this, SLOT(cancelJob()));
}
QString SettingsDialog::includesFileName(QString s) const
{
  QStringList fullPath = settings.fileName().split(QDir::separator());
  fullPath.removeLast();
  fullPath.append(QString("%1.included").arg(s));
  return fullPath.join(QDir::separator());
}
QString SettingsDialog::mountsFileName(QString s) const
{
  QStringList fullPath = settings.fileName().split(QDir::separator());
  fullPath.removeLast();
  fullPath.append(QString("%1.mounts").arg(s));
  return fullPath.join(QDir::separator());
}
void SettingsDialog::setJobItem(JobItemIndex *idx)
{
  own_index = idx;
  name = idx->getName();
  previousName = name;
  setWindowTitle(name);
  if (name!=QString("<noname>"))
    {
      w1->set_Job_Name(name);
      w3->set_FileName(includesFileName(name));
      w4->set_FileName(mountsFileName(name));
      initParameters();
    };
}
void SettingsDialog::saveJob()
{
  name = w1->get_Job_Name();
  w3->set_FileName(includesFileName(name));
  w4->set_FileName(mountsFileName(name));
  settings.beginGroup("Tasks");
  QStringList groups = settings.childGroups();
  settings.endGroup();
  if ( name.isEmpty() ) QMessageBox::information(this, QString("Info"), QString("JobName is empty."));
  else if ( groups.contains(name) && !newbe && name==previousName )
    {
      saveParameters();
      close();
    }
  else if ( groups.contains(name) && newbe )
    {
      QMessageBox::information(this, QString("Info"), QString("Same JobName is exist."));
    }
  else if ( groups.contains(name) && !newbe && name!=previousName )
    {
      QMessageBox::information(this, QString("Info"), QString("Same JobName is exist."));
    }
  else if ( !groups.contains(name) && newbe )
    {
      saveParameters();
      close();
    }
  else if ( !groups.contains(name) && !newbe )
    {
      settings.beginGroup("Tasks");
      settings.remove(previousName);
      settings.endGroup();
      QFile::remove(includesFileName(previousName));
      QFile::remove(mountsFileName(previousName));
      //qDebug()<<previousName<<"included deleted";
      saveParameters();
      close();
    };
}
void SettingsDialog::cancelJob()
{
  //qDebug()<<"cancel"<<name<< newbe;
  if ( newbe )
    {
      settings.beginGroup("Tasks");
      settings.remove(name);
      settings.endGroup();
      emit creatingJobCancelled();
    };
  close();
}
void SettingsDialog::initParameters()
{
  int c;
  settings.beginGroup("Tasks");
  QStringList groups = settings.childGroups();
  newbe = !groups.contains(name);
  //qDebug()<<"init"<<name<< newbe;
  settings.beginGroup(name);
  w1->autoRun->setChecked( settings.value("AutoRun", QVariant(false)).toBool() );
  w1->privileged->setChecked( settings.value("Privileged", QVariant(false)).toBool() );
  w1->shell->setChecked( settings.value("Shell", QVariant(false)).toBool() );
  w1->verbose->setChecked( settings.value("Verbose", QVariant(false)).toBool() );
  w1->debug->setChecked( settings.value("Debug", QVariant(false)).toBool() );
  c = w1->sandboxType->findText( settings.value("SType", QVariant("for QEMU")).toString() );
  if (c+1) w1->sandboxType->setCurrentIndex(c);
  w1->command->setText( settings.value("Command", QVariant()).toString() );
  w1->appendOption->setText( settings.value("AddOptions", QVariant()).toString() );
  w1->checkTimeout->setValue( settings.value("TimeOut", QVariant(TIMEOUT)).toInt() );
  w2->runInTerm->setChecked( settings.value("RunInTerminal", QVariant(true)).toBool() );
  w2->setCustomTerminalChecked( settings.value("CustomTerminal", QVariant(true)).toBool() );
  w2->setCustomTerminal( settings.value("Terminal", QVariant()).toString() );
  w3->enabled->setChecked( settings.value("EnableIncluded", QVariant(false)).toBool() );
  w3->setIncludesList( settings.value("Includes", QVariant()).toString() );
  w4->useMount->setChecked( settings.value("Mount", QVariant(false)).toBool() );
  w4->setCurrentRule( settings.value("MountBindRule", QVariant()).toString() );
  w5->useNetwork->setChecked( settings.value("Network", QVariant(false)).toBool() );
  w5->setCurrentType( settings.value("NetworkType", QVariant("dhcp")).toString() );
  w5->setCurrentAddress( settings.value("Address", QVariant()).toString() );
  w5->setCurrentSource( settings.value("Source", QVariant("default")).toString() );
  w5->setCurrentMAC( settings.value("MAC", QVariant()).toString() );
  w5->setCurrentRoute( settings.value("Route", QVariant()).toString() );
  w6->altSecurity->setChecked( settings.value("AltSecurity", QVariant(false)).toBool() );
  w6->setCurrentType( settings.value("AltSecurityType", QVariant("dynamic")).toString() );
  w6->setCurrentLabel( settings.value("AltSecSELabel", QVariant()).toString() );
  w7->setKernelForUse( settings.value("KernelForUse", QVariant()).toString() );
  settings.endGroup();
  settings.endGroup();
}
void SettingsDialog::saveParameters()
{
  settings.beginGroup("Tasks");
  settings.beginGroup(name);
  settings.setValue("AutoRun", QVariant(w1->autoRun->isChecked()));
  settings.setValue("Privileged", QVariant(w1->privileged->isChecked()));
  settings.setValue("Shell", QVariant(w1->shell->isChecked()));
  settings.setValue("Verbose", QVariant(w1->verbose->isChecked()));
  settings.setValue("Debug", QVariant(w1->debug->isChecked()));
  settings.setValue("SType", QVariant(w1->sandboxType->currentText()));
  settings.setValue("URI", w1->sandboxType->itemData(w1->sandboxType->currentIndex()));
  settings.setValue("Command", QVariant(w1->command->text()));
  settings.setValue("AddOptions", QVariant(w1->appendOption->text()));
  settings.setValue("TimeOut", QVariant(w1->checkTimeout->value()));
  settings.setValue("RunInTerminal", QVariant(w2->runInTerm->isChecked()));
  settings.setValue("CustomTerminal", QVariant(w2->getCustomTerminalChecked()));
  settings.setValue("Terminal", QVariant(w2->getCustomTerminal()));
  settings.setValue("EnableIncluded", QVariant(w3->enabled->isChecked()));
  settings.setValue("Includes", QVariant(w3->get_FileName()));
  settings.setValue("Mount", QVariant(w4->useMount->isChecked()));
  settings.setValue("MountBindRule", QVariant(w4->get_FileName()));
  settings.setValue("Network", QVariant(w5->useNetwork->isChecked()));
  settings.setValue("NetworkType", QVariant(w5->getCurrentType()));
  settings.setValue("Address", QVariant(w5->getCurrentAddress()));
  settings.setValue("Source", QVariant(w5->getCurrentSource()));
  settings.setValue("MAC", QVariant(w5->getCurrentMAC()));
  settings.setValue("Route", QVariant(w5->getCurrentRoute()));
  settings.setValue("AltSecurity", QVariant(w6->altSecurity->isChecked()));
  settings.setValue("AltSecurityType", QVariant(w6->getCurrentType()));
  settings.setValue("AltSecSELabel", QVariant(w6->getCurrentLabel()));
  settings.setValue("KernelForUse", QVariant(w7->getKernelForUse()));
  settings.endGroup();
  settings.endGroup();
  own_index->setName(name);
  //QString tT = item->toolTip();
  //tT.append("\n-- EDITED --");
  //item->setToolTip(tT);
  if ( own_index->getData().value("isRunning").toBool() )
    QMessageBox::information(this, "Info", "New settings apply\nat next job start.");
}

void SettingsDialog::closeEvent(QCloseEvent *ev)
{
  settings.setValue("SetDlgGeometry", saveGeometry());
  settings.sync();
  killTimer(timerId);
  ev->accept();
}

void SettingsDialog::set_Title_Name(QString s)
{
  setWindowTitle(s);
}

void SettingsDialog::set_Warning_Data(QString s)
{
    QString _warning;
    if ( "for QEMU"==s ) {
        _warning.append("<b>WARNING:</b><br>For it needed to disable SELinux.<br>This will be done automatically.");
        w7->setEnabled(true);
    } else {
        _warning.append("<b>WARNING:</b><br> <empty> ");
        w7->setEnabled(false);
    };
    warning->setText(_warning);
}

void SettingsDialog::timerEvent(QTimerEvent *event)
 {
   int _timerId = event->timerId();
   if ( _timerId && timerId==_timerId ) {
       QString sandboxType = w1->sandboxType->itemData(w1->sandboxType->currentIndex()).toString();
       bool privileged = w1->privileged->isChecked();
       bool shell = w1->shell->isChecked();
       bool verbose = w1->verbose->isChecked();
       bool debug = w1->debug->isChecked();
       bool useMount = w4->useMount->isChecked();
       QString mountType = w4->getCurrentType();
       QStringList bindRules = w4->getCurrentRules();
       bool enableIncluded = w3->enabled->isChecked();
       QString includes = w3->fileName;
       bool enableNetwork = w5->useNetwork->isChecked();
       QString netType = w5->getCurrentType();
       QString address = w5->getCurrentAddress();
       QString sources = w5->getCurrentSource();
       QString macAddr = w5->getCurrentMAC();
       QString route = w5->getCurrentRoute();
       bool altSecurity = w6->altSecurity->isChecked();
       QString securityType = w6->getCurrentType();
       QString securitySELabel = w6->getCurrentLabel();
       QString kernelForUse = w7->getKernelForUse();
       QString command = w1->command->text();
       QString addOption = w1->appendOption->text();
       // command building
       QStringList commandLine;
       if (!command.isEmpty()) {
           // append connectURI
           commandLine.append("-c");
           commandLine.append(sandboxType);
           // append Name
           commandLine.append("-n");
           commandLine.append(name);
           if (useMount) {
               // append mount options
               if ( !bindRules.isEmpty() ) {
                   commandLine.append("-m");
                   QString s = bindRules.join(" -m ");
                   commandLine.append(s);
               };
           };
           if (enableIncluded) {
               // append includes options
               commandLine.append("-I");
               commandLine.append(includes);
           };
           if (enableNetwork) {
               // append network options
               QString networkOpt;
               if ( netType.toLower()!="dhcp" ) {
                   netType.append("=");
                   netType.append(address);
               };
               networkOpt.append(netType);
               if ( !sources.isEmpty() ) {
                   networkOpt.append(",");
                   networkOpt.append("source=");
                   networkOpt.append(sources);
               };
               if ( !macAddr.isEmpty() ) {
                   networkOpt.append(",");
                   networkOpt.append("mac=");
                   networkOpt.append(macAddr);
               };
               if ( !route.isEmpty() ) {
                   networkOpt.append(",");
                   networkOpt.append("route=");
                   networkOpt.append(route);
               };
               commandLine.append("-N");
               commandLine.append(networkOpt);
           };
           if (altSecurity) {
               //append security options
               QString s;
               if (securityType=="dynamic" || securityType=="inherit") s = securityType;
               else s = QString("%1=%2").arg(securityType).arg(securitySELabel);
               commandLine.append("-s");
               commandLine.append(s);
           };
           // append kernel options (QEMU only)
           if ( sandboxType.startsWith("qemu:///") && !kernelForUse.isEmpty() ) {
               QString version = kernelForUse.section(KERNEL, -1);
               commandLine.append(QString("--kernver=%1").arg(version));
               commandLine.append(QString("--kernpath=%1").arg(kernelForUse));
           };
           // append additional options
           if (!addOption.isEmpty()) commandLine.append(addOption);
           // append priveleged
           if (privileged) commandLine.append("-p");
           //append shell
           if (shell) commandLine.append("-l");
           // append verbose
           if (verbose) commandLine.append("-v");
           // append debug
           if (debug) commandLine.append("-d");
           // append command
           commandLine.append(command);
           // prepend virt-sandbox command
           commandLine.prepend(QString("virt-sandbox"));
           // prepend terminal if specified
           if ( w2->runInTerm->isChecked() ) {
               if ( !w2->getCustomTerminalChecked() ) {
                   commandLine.prepend(QString("xdg-terminal-util"));
               }
               else if ( !w2->getCustomTerminal().isEmpty() ) {
                   QStringList _terminal;
                   foreach ( QString s, w2->getCustomTerminal().split(" ") ) {
                       if ( !s.isEmpty() ) _terminal.prepend(s);
                   };
                   foreach ( QString s, _terminal ) {
                       commandLine.prepend(s);
                   };
               }
               else {
                   commandLine.prepend(QString("-e"));
                   commandLine.prepend(QString("xterm"));
               };
           };
       };
       fullCommandWdg->setText(commandLine.join(" "));
     };
 }

void SettingsDialog::tabChanged(int i)
{
    switch (i) {
    case 0: {
        QString s = w1->sandboxType->currentText();
        set_Warning_Data(s);
        break;
    }
    case 1: {
        warning->setText("<b>WARNING:</b><br>If your terminal create the detached processes,<br>then you must to kill Job process' children<br>manually.");
        break;
    }
    case 2: {
        warning->setText("<b>WARNING:</b><br>Parses includefile whose elements are in the format<br>GUEST-TARGET=ROOT-PATH. If ROOT_PATH is omitted,<br>then it is assumed to be the same as GUEST-TARGET.<br>Mounting of work directories must to be enabled.");
        break;
    }
    case 3: {
        /*
         * https://www.redhat.com/archives/libvirt-users/2013-December/msg00052.html
         */
        warning->setText("<b>WARNING:</b><br>The mounts will be processed in the order<br>which you specify in the list. So eg, you should specify<br>a mount at /var before one at /var/log.<br>Otherwise your /var mount will hide the /var/log contents");
        break;
    }
    case 4: {
        warning->setText("<b>WARNING:</b><br> <a href=\"http://libvirt.org/formatnetwork.html\">See for libvirt virtual network</a>.<br>See also <b>VirtNetwork Control</b>");
        break;
    }
    case 5: {
        warning->setText("<b>WARNING:</b><br> <empty> ");
        break;
    }
    case 6: {
        struct utsname UTSNAME;
        QString kernelRelease;
        if ( !uname(&UTSNAME) ) {
            kernelRelease.append(UTSNAME.release);
        } else {
            kernelRelease.append("Undefined");
        };
        warning->setText(QString("<b>WARNING:</b><br> System Kernel Release: %1").arg(kernelRelease));
        break;
    }
    default:
        break;
    };
}

void SettingsDialog::showVirtControl()
{
    emit showVirtNetControl();
}
