#ifndef SETTINGS_H
#define SETTINGS_H

#include <QDebug>
#include <QDialog>
#include <QTabWidget>
#include <QPushButton>
#include <QModelIndex>
#include <QTextEdit>
#include <QSettings>
#include <QCloseEvent>
#include <QProcess>
#include "common_settings.h"
#include "terminal_set.h"
#include "include_settings.h"
#include "directory_settings.h"
#include "network_settings.h"
#include "security_settings.h"
#include "kernel_set.h"
#include "sys/utsname.h"
#include "layout/jobitem_model.h"

class SettingsDialog : public QDialog
{
    Q_OBJECT;
public:
    SettingsDialog(QWidget *parent=0);
    ~SettingsDialog();
    QSettings    settings;
    QTabWidget  *tabWidget;

signals:
    void creatingJobCancelled();
    void showVirtNetControl();

private:
    JobItemIndex        *own_index;

    QVBoxLayout         *commonLayout;
    QHBoxLayout         *buttonsLayout;
    QWidget             *buttons;
    QListWidgetItem     *item;
    QString              name;
    QPushButton         *ok;
    QPushButton         *cancel;
    CommonSet           *w1;
    TerminalSet         *w2;
    IncludeSet          *w3;
    MountSet            *w4;
    NetworkSet          *w5;
    SecuritySet         *w6;
    KernelSet           *w7;
    bool                 newbe;
    QString              previousName;
    QLabel              *warning;
    QTextEdit           *fullCommandWdg;
    int                  timerId;

public slots:
    void setJobItem(JobItemIndex*);

private slots:
    void initTabWidget();
    void initButtons();
    QString includesFileName(QString) const;
    QString mountsFileName(QString) const;
    void initParameters();
    void saveJob();
    void cancelJob();
    void saveParameters();
    void closeEvent(QCloseEvent*);
    void set_Title_Name(QString);
    void set_Warning_Data(QString);
    void timerEvent(QTimerEvent*);
    void tabChanged(int);
    void showVirtControl();
};

#endif
