#include "terminal_set.h"

TerminalSet::TerminalSet(QWidget *parent) :
    QWidget(parent)
{
    setContentsMargins(1, 1, 1, 1);
    termChoiseLayout = new QVBoxLayout();
    runInTerm = new QCheckBox("Run in Terminal:", this);
    defaultTerminal = new QRadioButton("use xdg-terminal util", this);
    customTerminal = new QRadioButton("use custom terminal command\n(Recommended)", this);
    termCommand = new QLineEdit(this);
    termCommand->setEnabled(false);
    termCommand->setPlaceholderText("konsole --nofork -e");
    termChoiseLayout->addWidget(runInTerm);
    termChoiseLayout->addWidget(defaultTerminal);
    termChoiseLayout->addWidget(customTerminal);
    termChoiseLayout->addWidget(termCommand);
    termChoiseLayout->insertStretch(-1);
    setLayout(termChoiseLayout);
    connect(runInTerm, SIGNAL(toggled(bool)), this, SLOT(enableWidgets(bool)));
    connect(customTerminal, SIGNAL(toggled(bool)), termCommand, SLOT(setEnabled(bool)));
}
TerminalSet::~TerminalSet()
{
    disconnect(runInTerm, SIGNAL(toggled(bool)), this, SLOT(enableWidgets(bool)));
    disconnect(customTerminal, SIGNAL(toggled(bool)), termCommand, SLOT(setEnabled(bool)));

    delete runInTerm;
    runInTerm = 0;
    delete defaultTerminal;
    defaultTerminal = 0;
    delete customTerminal;
    customTerminal = 0;
    delete termCommand;
    termCommand = 0;
    delete termChoiseLayout;
    termChoiseLayout = 0;
}
void TerminalSet::enableWidgets(bool b)
{
    defaultTerminal->setEnabled(b);
    customTerminal->setEnabled(b);
    termCommand->setEnabled( b && customTerminal->isChecked() );
}
void TerminalSet::setCustomTerminalChecked(bool b)
{
    if (b) customTerminal->setChecked(b);
    else defaultTerminal->setChecked(!b);
}
bool TerminalSet::getCustomTerminalChecked() const
{
    return customTerminal->isChecked();
}
void TerminalSet::setCustomTerminal(QString s)
{
    termCommand->setText(s);
}
QString TerminalSet::getCustomTerminal() const
{
    return termCommand->text();
}
