#include "auth_dialog.h"

AuthMenu::AuthMenu(QMenu *parent) :
    QMenu(parent)
{
    authType = QString();
    setYes_Action = addAction("Authorized");
    setYes_Action->setIcon(QIcon::fromTheme("security-low"));
    setYes_Action->setCheckable(true);
    setYes_Action->setData("yes");
    setAuthSelf_Action = addAction("Self authorization");
    setAuthSelf_Action->setIcon(QIcon::fromTheme("security-medium"));
    setAuthSelf_Action->setCheckable(true);
    setAuthSelf_Action->setData("auth_self");
    setAuthAdmin_Action = addAction("Admin authorization");
    setAuthAdmin_Action->setIcon(QIcon::fromTheme("security-high"));
    setAuthAdmin_Action->setCheckable(true);
    setAuthAdmin_Action->setData("auth_admin");
    setAuthSelfKeep_Action = addAction("Self Keep authorization");
    setAuthSelfKeep_Action->setIcon(QIcon::fromTheme("security-medium-timed"));
    setAuthSelfKeep_Action->setCheckable(true);
    setAuthSelfKeep_Action->setData("auth_self_keep");
    setAuthAdminKeep_Action = addAction("Admin Keep authorization");
    setAuthAdminKeep_Action->setIcon(QIcon::fromTheme("security-high-timed"));
    setAuthAdminKeep_Action->setCheckable(true);
    setAuthAdminKeep_Action->setData("auth_admin_keep");
    setToolTips();
    actionGroup = new QActionGroup(this);
    actionGroup->addAction(setYes_Action);
    actionGroup->addAction(setAuthSelf_Action);
    actionGroup->addAction(setAuthAdmin_Action);
    actionGroup->addAction(setAuthSelfKeep_Action);
    actionGroup->addAction(setAuthAdminKeep_Action);
    actionGroup->setExclusive(true);
    connectToSLOTs();
    helperCaller = new HelperCaller(this);
    connect(helperCaller, SIGNAL(resultData(QStringList&)), this, SLOT(saveAuthType(QStringList&)));
}
AuthMenu::~AuthMenu()
{
    disconnectFromSLOTs();
    disconnect(helperCaller, SIGNAL(resultData(QStringList&)), this, SLOT(saveAuthType(QStringList&)));

    delete setYes_Action;
    setYes_Action = 0;
    delete setAuthSelf_Action;
    setAuthSelf_Action = 0;
    delete setAuthAdmin_Action;
    setAuthAdmin_Action = 0;
    delete setAuthSelfKeep_Action;
    setAuthSelfKeep_Action = 0;
    delete setAuthAdminKeep_Action;
    setAuthAdminKeep_Action = 0;
    delete actionGroup;
    actionGroup = 0;
    delete helperCaller;
    helperCaller = 0;
}
void AuthMenu::initAuthType()
{
    QStringList args;
    args<<"getAuthType"<<objectName();
    bool result = helperCaller->actionActivated(args);
}
void AuthMenu::setCurrentAuthType()
{
    settings.beginGroup("AuthType");
    authType = settings.value(objectName()).toString();
    settings.endGroup();
    setCurrentIcon(authType);
}
void AuthMenu::setCurrentIcon(const QString &authType)
{
    disconnectFromSLOTs();
    foreach ( QAction *action, actionGroup->actions() ) {
        if (action->data().toString()==authType) {
            action->setChecked(true);
            QString iconTheme;
            if (authType=="yes") iconTheme = "security-low";
            else if (authType=="auth_self") iconTheme = "security-medium";
            else if (authType=="auth_admin") iconTheme = "security-high";
            else if (authType=="auth_self_keep") iconTheme = "security-medium-timed";
            else if (authType=="auth_admin_keep") iconTheme = "security-high-timed";
            else iconTheme = "security-low";
            emit currentIcon(iconTheme);
        };
    };
    connectToSLOTs();
}
void AuthMenu::connectToSLOTs()
{
    connect(setYes_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    connect(setAuthSelf_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    connect(setAuthAdmin_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    connect(setAuthSelfKeep_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    connect(setAuthAdminKeep_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
}
void AuthMenu::disconnectFromSLOTs()
{
    disconnect(setYes_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    disconnect(setAuthSelf_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    disconnect(setAuthAdmin_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    disconnect(setAuthSelfKeep_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
    disconnect(setAuthAdminKeep_Action, SIGNAL(toggled(bool)), this, SLOT(setAuthType(bool)));
}
void AuthMenu::setToolTips()
{
    /*
     * http://www.freedesktop.org/software/polkit/docs/latest/polkit.8.html
     */
    setYes_Action->setToolTip("Authorized.");
    setAuthSelf_Action->setToolTip("Authentication by the owner of the session\nthat the client originates from is required.\nNote that this is not restrictive enough\nfor most uses on multi-user systems;\nauth_admin* is generally recommended.");
    setAuthAdmin_Action->setToolTip("Authentication by an administrative user is required.");
    setAuthSelfKeep_Action->setToolTip("Like auth_self but the authorization is kept\nfor a brief period (e.g. five minutes).\nThe warning about auth_self above applies likewise.");
    setAuthAdminKeep_Action->setToolTip("Like auth_admin but the authorization is kept\nfor a brief period (e.g. five minutes).");
}
void AuthMenu::setAuthType(bool b)
{
    if (!b) return ;
    QAction *action = qobject_cast<QAction *>(sender());
    authType = action->data().toString();
    //qDebug()<<authType;
    if (authType.isEmpty()) return;
    QStringList args;
    args<<"setAuthType"<<objectName()<<authType;
    bool result = helperCaller->actionActivated(args);
    //qDebug()<<objectName()<<authType<<" authority: "<<result;
    if ( !result ) {
        /* Action cancelled or broken */
        setCurrentAuthType();
    };
}
void AuthMenu::saveAuthType(QStringList &s)
{
    if ( !s.isEmpty() ) {
        if ( QVariant(s.first()).toBool() ) {
            if ( authType.isEmpty() || authType!=s.last() ) {
                QStringList AuthTypes;
                AuthTypes<< "yes" << "auth_self" << "auth_admin" << "auth_self_keep" << "auth_admin_keep";
                if ( AuthTypes.contains(s.last()) ) authType = s.last();
                else authType = "auth_admin";
            };
            //qDebug()<<s<<"save";
            settings.beginGroup("AuthType");
            settings.setValue(objectName(), authType);
            settings.endGroup();
            settings.sync();
        }
        else {
            QMessageBox::warning(0, "Set AuthType", s.last());
            emit warningShowed();
        };
    };
    setCurrentAuthType();
}
