#include "toolbar.h"

ToolBar::ToolBar (QWidget *parent = 0) : QToolBar(parent)
{
  setAllowedAreas(Qt::AllToolBarAreas);
  setMovable(true);
  setFloatable(true);

  initActions();
}
ToolBar::~ToolBar()
{
  disconnect(itemControlAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
  disconnect(authTypeAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
  disconnect(_hideAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
  disconnect(_virtNetAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
  disconnect(_exitAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));

  disconnect(startMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  disconnect(stopMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  disconnect(setSELinuxModeMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  disconnect(getNetControlMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  disconnect(setAuthTypeMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  disconnect(startMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  disconnect(stopMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  disconnect(setSELinuxModeMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  disconnect(getNetControlMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  disconnect(setAuthTypeMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));

  delete start_Action;
  start_Action = 0;
  delete stop_Action;
  stop_Action = 0;
  delete setSELinuxMode_Action;
  setSELinuxMode_Action = 0;
  delete getNetControl_Action;
  getNetControl_Action = 0;
  delete setAuthType_Action;
  setAuthType_Action = 0;

  delete startMethodMenu;
  startMethodMenu = 0;
  delete stopMethodMenu;
  stopMethodMenu = 0;
  delete setSELinuxModeMethodMenu;
  setSELinuxModeMethodMenu = 0;
  delete getNetControlMethodMenu;
  getNetControlMethodMenu = 0;
  delete setAuthTypeMethodMenu;
  setAuthTypeMethodMenu = 0;

  delete authTypeMenu;
  authTypeMenu = 0;
  delete authTypeAction;
  authTypeAction = 0;

  delete _hideAction;
  _hideAction = 0;
  delete _virtNetAction;
  _virtNetAction = 0;
  delete _exitAction;
  _exitAction = 0;

  delete _createAction;
  _createAction = 0;
  delete _deleteAction;
  _deleteAction = 0;
  delete _editAction;
  _editAction = 0;
  delete _runAction;
  _runAction = 0;
  delete _undockAction;
  _undockAction = 0;
  delete _stopAction;
  _stopAction = 0;
  delete _stopAllAction;
  _stopAllAction = 0;
  delete itemControlMenu;
  itemControlMenu = 0;
  delete itemControlAction;
  itemControlAction = 0;
}
void ToolBar::initActions()
{
    _hideAction = new QAction(QString("Hide to tray"), this);
    _hideAction->setIcon ( QIcon::fromTheme("down") );
    _virtNetAction = new QAction(QString("Virtual Network Control"), this);
    _virtNetAction->setIcon ( QIcon::fromTheme("preferences-system-network") );
    _exitAction = new QAction(QString("Exit"), this);
    _exitAction->setIcon ( QIcon::fromTheme("exit") );

    addAction(_hideAction);
    addSeparator();
    addItemControlMenu();
    addSeparator();
    addAuthTypeMenu();
    addSeparator();
    addAction(_virtNetAction);
    addSeparator();
    addAction(_exitAction);

    connect(_hideAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
    connect(_virtNetAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
    connect(_exitAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
}
void ToolBar::addItemControlMenu()
{
    itemControlMenu = new QMenu(this);

    _createAction = new QAction(QString("Create new Job"), this);
    _createAction->setIcon ( QIcon::fromTheme("install") );
    _editAction = new QAction(QString("Edit selected Job"), this);
    _editAction->setIcon ( QIcon::fromTheme("configure") );
    _deleteAction = new QAction(QString("Delete selected Job"), this);
    _deleteAction->setIcon ( QIcon::fromTheme("clean") );
    _runAction = new QAction(QString("Run selected Job"), this);
    _runAction->setIcon ( QIcon::fromTheme("run") );
    _undockAction = new QAction(QString("Undock selected Job"), this);
    _undockAction->setIcon ( QIcon::fromTheme("undock") );
    _stopAction = new QAction(QString("Kill selected Job"), this);
    _stopAction->setIcon ( QIcon::fromTheme("stop") );
    _stopAllAction = new QAction(QString("Kill all Job"), this);
    _stopAllAction->setIcon ( QIcon::fromTheme("stop-all") );

    itemControlMenu->addAction(_createAction);
    itemControlMenu->addAction(_editAction);
    itemControlMenu->addAction(_deleteAction);
    itemControlMenu->addSeparator();
    itemControlMenu->addAction(_runAction);
    itemControlMenu->addAction(_undockAction);
    itemControlMenu->addAction(_stopAction);
    itemControlMenu->addAction(_stopAllAction);

    itemControlAction = new QAction(this);
    itemControlAction->setIcon(QIcon::fromTheme("job"));
    itemControlAction->setToolTip("Job Control");
    itemControlAction->setMenu(itemControlMenu);
    addAction( itemControlAction );
    connect(itemControlAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));
}
void ToolBar::addAuthTypeMenu()
{
  authTypeMenu = new QMenu(this);

  startMethodMenu = new AuthMenu(authTypeMenu);
  startMethodMenu->setObjectName("start");
  start_Action = authTypeMenu->addMenu(startMethodMenu);
  start_Action->setText("Start Action");

  stopMethodMenu = new AuthMenu(authTypeMenu);
  stopMethodMenu->setObjectName("stop");
  stop_Action = authTypeMenu->addMenu(stopMethodMenu);
  stop_Action->setText("Stop Action");

  setSELinuxModeMethodMenu = new AuthMenu(authTypeMenu);
  setSELinuxModeMethodMenu->setObjectName("setCurrentSELinuxMode");
  setSELinuxMode_Action = authTypeMenu->addMenu(setSELinuxModeMethodMenu);
  setSELinuxMode_Action->setText("SetSELinuxMode Action");

  getNetControlMethodMenu = new AuthMenu(authTypeMenu);
  getNetControlMethodMenu->setObjectName("getNetworkControlAccess");
  getNetControl_Action = authTypeMenu->addMenu(getNetControlMethodMenu);
  getNetControl_Action->setText("GetNetworkControl Action");

  authTypeMenu->addSeparator();

  setAuthTypeMethodMenu = new AuthMenu(authTypeMenu);
  setAuthTypeMethodMenu->setObjectName("setAuthType");
  setAuthType_Action = authTypeMenu->addMenu(setAuthTypeMethodMenu);
  setAuthType_Action->setText("SetAuthType Action");

  authTypeAction = new QAction(this);
  authTypeAction->setText("Authentification Type");
  authTypeAction->setIcon(QIcon::fromTheme("security-high"));
  authTypeAction->setToolTip("Set Authentification Type");
  authTypeAction->setMenu(authTypeMenu);
  addAction(authTypeAction);
  connect(authTypeAction, SIGNAL(hovered()), this, SLOT(showHoveredMenu()));

  connect(startMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  connect(stopMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  connect(setSELinuxModeMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  connect(getNetControlMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  connect(setAuthTypeMethodMenu, SIGNAL(currentIcon(const QString&)), this, SLOT(setMenuIcon(const QString&)));
  connect(startMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  connect(stopMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  connect(setSELinuxModeMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  connect(getNetControlMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));
  connect(setAuthTypeMethodMenu, SIGNAL(warningShowed()), this, SLOT(mainWindowUp()));

  startMethodMenu->initAuthType();
  stopMethodMenu->initAuthType();
  setSELinuxModeMethodMenu->initAuthType();
  getNetControlMethodMenu->initAuthType();
  setAuthTypeMethodMenu->initAuthType();
}
void ToolBar::setMenuIcon(const QString &iconName)
{
    AuthMenu *menu = qobject_cast<AuthMenu*>(sender());
    menu->setIcon(QIcon::fromTheme(iconName));
}
void ToolBar::mainWindowUp()
{
    emit warningShowed();
}
void ToolBar::showHoveredMenu()
{
    QAction *act = static_cast<QAction*>(sender());
    if ( act==authTypeAction ) {
        if ( itemControlAction->menu()->isVisible() )
            itemControlAction->menu()->hide();
    } else if ( act==itemControlAction ) {
        if ( authTypeAction->menu()->isVisible() )
            authTypeAction->menu()->hide();
    } else {
        if ( itemControlAction->menu()->isVisible() )
            itemControlAction->menu()->hide();
        if ( authTypeAction->menu()->isVisible() )
            authTypeAction->menu()->hide();
        return;
    };
    //act->menu()->popup(mapToGlobal(this->pos()), act);
    act->menu()->show();
    act->menu()->move(QCursor::pos());
}

Qt::ToolBarArea ToolBar::get_ToolBarArea(int i) const
{
  Qt::ToolBarArea result;
  switch (i) {
  case 1:
    result = Qt::LeftToolBarArea;
    break;
  case 2:
    result = Qt::RightToolBarArea;
    break;
  case 4:
    result = Qt::TopToolBarArea;
    break;
  case 8:
    result = Qt::BottomToolBarArea;
    break;
  default:
    result = Qt::TopToolBarArea;
    break;
  };
  return result;
}
