#include "doc_update_content.h"

DocUpdateContent::DocUpdateContent(QWidget *parent, bool check, int val) :
    QWidget(parent), checkList(check), lastInterval(val)
{
    commonLayout = new QVBoxLayout();
    updateLayout = new QHBoxLayout();
    virtNetLabel = new QLabel("Update List:", this);
    activeIcon = QIcon::fromTheme("ledlightred");
    usualIcon = QIcon::fromTheme("leddarkred");
    no_activeIcon = QIcon::fromTheme("ledlightblue");
    LED = new QPushButton(this);
    LED->setIcon(usualIcon);
    LED->setCheckable(true);
    LED->setChecked(checkList);
    changeUpdateState(checkList);
    connect(LED, SIGNAL(clicked(bool)), this, SLOT(changeUpdateState(bool)));
    interval = new QSpinBox(this);
    interval->setRange(2,120);
    interval->setSingleStep(1);
    //interval->setObjectName("IntervalSpinBox");
    //lastInterval = interval->value();
    interval->setValue(lastInterval);
    connect(interval, SIGNAL(valueChanged(int)), this, SLOT(editInterval()));
    applyInterval = new QPushButton(QIcon::fromTheme("dialog-ok"), "", this);
    applyInterval->setVisible(false);
    //applyInterval->setObjectName("ApplyButton");
    connect(applyInterval, SIGNAL(clicked()), this, SLOT(setNewInterval()));
    errorLog = new QTextEdit(this);
    errorLog->setToolTip("VirtNet Error Log");
    errorLog->setReadOnly(true);
    errorLog->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(errorLog, SIGNAL(customContextMenuRequested(const QPoint&)), errorLog, SLOT(clear()));

    updateLayout->addWidget(LED);
    updateLayout->addWidget(virtNetLabel);
    updateLayout->addWidget(applyInterval);
    updateLayout->addWidget(interval);
    commonLayout->addItem(updateLayout);
    commonLayout->addWidget(errorLog);
    setLayout(commonLayout);

    updateInterval = 0;
    startUpdateTimer();
    applyTimer = new QTimer(this);
    applyTimer->setInterval(3000);
    //applyTimer->setObjectName("ApplyTimer");
    connect(applyTimer, SIGNAL(timeout()), this, SLOT(editIntervalDown()));
}
DocUpdateContent::~DocUpdateContent()
{
    stopUpdateTimer();
    delete virtNetLabel;
    virtNetLabel = 0;
    if ( LED!=NULL ) {
        disconnect(LED, SIGNAL(clicked(bool)), this, SLOT(changeUpdateState(bool)));
        delete LED;
        LED = 0;
    };
    if ( interval!=NULL ) {
        disconnect(interval, SIGNAL(valueChanged(int)), this, SLOT(editInterval()));
        delete interval;
        interval = 0;
    };
    if ( applyTimer!=NULL ) {
        disconnect(applyTimer, SIGNAL(timeout()), this, SLOT(editIntervalDown()));
        delete applyTimer;
        applyTimer = 0;
    };
    if ( applyInterval!=NULL ) {
        disconnect(applyInterval, SIGNAL(clicked()), this, SLOT(setNewInterval()));
        delete applyInterval;
        applyInterval = 0;
    };
    disconnect(errorLog, SIGNAL(customContextMenuRequested(const QPoint&)), errorLog, SLOT(clear()));
    errorLog->clear();
    delete errorLog;
    errorLog = 0;
    commonLayout->removeItem(updateLayout);
    if ( updateLayout!=NULL ) {
        delete updateLayout;
        updateLayout = 0;
    };
    delete commonLayout;
    commonLayout = 0;
}
void DocUpdateContent::timerEvent(QTimerEvent *event)
{
    int _timerId = event->timerId();
    if ( _timerId && updateInterval==_timerId ) {
        if ( LED->isChecked() ) {
            QStringList args;
            args.append("getVirtNetList");
            emit actionActivated(args);
        };
    };
}
void DocUpdateContent::setUsualLEDState()
{
    LED->setIcon(usualIcon);
}
void DocUpdateContent::editInterval()
{
    if ( updateInterval ) {
        // TODO: disable control buttons & menu
        // stop update NetList timer
        stopUpdateTimer();
        applyInterval->setVisible(true);
        applyTimer->start();
    };
}
void DocUpdateContent::setNewInterval()
{
    //qDebug()<<"new interval set";
    lastInterval = interval->value();
    editIntervalDown();
}
void DocUpdateContent::editIntervalDown()
{
    //qDebug()<<QString(sender()->objectName());
    if ( sender()==applyTimer && interval->hasFocus() ) {
        applyTimer->start();
        return;
    };
    applyTimer->stop();
    applyInterval->setVisible(false);
    interval->setValue(lastInterval);
    // TODO: enable control buttons & menu
    // start update NetList timer
    updateInterval = startTimer(lastInterval*1000);
}
void DocUpdateContent::changeUpdateState(bool b)
{
    QString state;
    if ( b ) {
        state.append("Enabled");
        //qDebug()<<"on"<<LED->isChecked();
    } else {
        state.append("Disabled");
        //qDebug()<<"off"<<LED->isChecked();
    };
    LED->setToolTip(QString("Update VirtNetwork List is %1").arg(state));
    checkList = b;
}
void DocUpdateContent::setLEDActiveIcon()
{
    LED->setIcon(activeIcon);
    QTimer::singleShot(333, this, SLOT(setUsualLEDState()));
}
void DocUpdateContent::stopUpdateTimer()
{
    if (updateInterval) {
        killTimer(updateInterval);
        updateInterval = 0;
    };
}
void DocUpdateContent::startUpdateTimer()
{
    if ( updateInterval==0 ) updateInterval = startTimer(lastInterval*1000);
}
int DocUpdateContent::getUpdateTime() const
{
    return lastInterval;
}
bool DocUpdateContent::getCheckList() const
{
    return checkList;
}
void DocUpdateContent::appendErrorMsg(QString &msg)
{
    if ( errorLog->toPlainText().count()>10240 ) errorLog->clear();
    errorLog->append(msg);
}
