#include "virtnet_control.h"

VirtNetControl::VirtNetControl(QWidget *parent) :
    QMainWindow(parent)
{
    //setObjectName("VirtNetContriol");
    setWindowTitle("VirtNet Control");
    setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
    setContentsMargins(0, 0, 0, 5);
    setWindowIcon(QIcon::fromTheme("preferences-system-network"));
    central = new QWidget(this);
    commonLayout = new QGridLayout();
    virtNetModel = new VirtNetModel();
    virtNetList = new QTreeView(this);
    virtNetList->setItemsExpandable(false);
    virtNetList->setRootIsDecorated(false);
    virtNetList->setModel(virtNetModel);
    virtNetList->setFocus();
    virtNetList->setContextMenuPolicy(Qt::CustomContextMenu);
    //connect(virtNetList, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(networkDoubleClicked(const QModelIndex&)));
    connect(virtNetList, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(networkClicked(const QPoint&)));
    commonLayout->addWidget(virtNetList, 2, 0, 3, 6);
    callThread = new CallThread(this);
    connect(callThread, SIGNAL(resultData(QStringList)), this, SLOT(resultReceiver(QStringList)));
    central->setLayout(commonLayout);
    setCentralWidget(central);
    initDockWidget();
    setMouseTracking(true);
    settings.beginGroup("VirtNetControl");
    restoreGeometry(settings.value("Geometry").toByteArray());
    virtNetList->setColumnWidth(0, settings.value("column0", 132).toInt());
    virtNetList->setColumnWidth(1, settings.value("column1", 32).toInt());
    virtNetList->setColumnWidth(2, settings.value("column2", 32).toInt());
    virtNetList->setColumnWidth(3, settings.value("column3", 32).toInt());
    int area_int = settings.value("ToolBarArea", 4).toInt();
    settings.endGroup();
    toolBar = new VirtNetToolBar(this);
    addToolBar(toolBar->get_ToolBarArea(area_int), toolBar);
    connect(toolBar, SIGNAL(fileForMethod(const QStringList&)), this, SLOT(actionActivated(const QStringList&)));
    connect(toolBar, SIGNAL(execMethod(const QStringList&)), this, SLOT(appendNetworkName(const QStringList&)));
}
VirtNetControl::~VirtNetControl()
{
    settings.beginGroup("VirtNetControl");
    settings.setValue("Geometry", saveGeometry());
    settings.setValue("column0", virtNetList->columnWidth(0));
    settings.setValue("column1", virtNetList->columnWidth(1));
    settings.setValue("column2", virtNetList->columnWidth(2));
    settings.setValue("column3", virtNetList->columnWidth(3));
    settings.setValue("UpdateTime", docContent->getUpdateTime());
    settings.setValue("CheckList", docContent->getCheckList());
    settings.setValue("ToolBarArea", toolBarArea(toolBar));
    settings.endGroup();
    settings.sync();
    //disconnect(virtNetList, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(networkDoubleClicked(const QModelIndex&)));
    disconnect(virtNetList, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(networkClicked(const QPoint&)));
    disconnect(docContent, SIGNAL(actionActivated(const QStringList&)), this, SLOT(actionActivated(const QStringList&)));
    disconnect(toolBar, SIGNAL(fileForMethod(const QStringList&)), this, SLOT(actionActivated(const QStringList&)));
    disconnect(toolBar, SIGNAL(execMethod(const QStringList&)), this, SLOT(appendNetworkName(const QStringList&)));
    disconnect(callThread, SIGNAL(resultData(QStringList)), this, SLOT(resultReceiver(QStringList)));
    if (callThread!=NULL) {
        callThread->stop();
        delete callThread;
        callThread = 0;
    };
    delete toolBar;
    toolBar = 0;

    if (virtNetModel!=NULL) {
        delete virtNetModel;
        virtNetModel = 0;
    };

    if (virtNetList!=NULL) {
        delete virtNetList;
        virtNetList = 0;
    };
    if ( docContent!=NULL ) {
        delete docContent;
        docContent = 0;
    };
    if ( doc!=NULL ) {
        delete doc;
        doc = 0;
    };
    delete commonLayout;
    commonLayout = 0;
    delete central;
    central = 0;
}
void VirtNetControl::initDockWidget()
{
    doc = new DocUpdate(this);
    settings.beginGroup("VirtNetControl");
    bool check = settings.value("CheckList", false).toBool();
    int interval = settings.value("UpdateTime", 3).toInt();
    settings.endGroup();
    docContent = new DocUpdateContent(doc, check, interval);
    doc->setWidget(docContent);
    addDockWidget(Qt::BottomDockWidgetArea, doc);
    connect(docContent, SIGNAL(actionActivated(const QStringList&)), this, SLOT(actionActivated(const QStringList&)));
}
void VirtNetControl::closeEvent(QCloseEvent *ev)
{
    settings.beginGroup("VirtNetControl");
    settings.setValue("Geometry", saveGeometry());
    settings.endGroup();
    settings.sync();
    if (callThread!=NULL) {
        callThread->stop();
    };
    docContent->stopUpdateTimer();
    close();
}
bool VirtNetControl::actionActivated(const QStringList &args)
{
    if (args.isEmpty()) return false;
    //qDebug() << "pretending to be the mechanism for action:" << args.first();

    if ( args.first()=="getVirtNetList" && !docContent->getCheckList() ) return false;

    Authority::Result result;
    UnixProcessSubject subject(static_cast<uint>(QCoreApplication::applicationPid()));

    result = Authority::instance()->checkAuthorizationSync(QString("%1.%2").arg(DOMAIN).arg(args.first()), subject,
             Authority::AllowUserInteraction);
    bool auth = (result == Authority::Yes);
    //qDebug() << "caller is"<< (( auth )? "" : "not") << "authorized to do:" << args.first();
    if ( auth ) {
        QStringList _args = args;
        if ( !callThread->isRunning() ) {
            callThread->setArgs(_args, 60);
            callThread->start();
            docContent->setLEDActiveIcon();
        }
        else warningShow("Authoring", "Process is busy.\nTry later.");
    }
    else warningShow("Authoring",
        QString("Not authorized to do %1.").arg(args.first()));
    return auth;
}
void VirtNetControl::resultReceiver(QStringList data)
{
    QString _result = data.first();
    data.removeFirst();
    QString method = data.first();
    data.removeFirst();
    //qDebug()<<_result<<method<<data<<"result";
    if ( callThread->isRunning() ) callThread->stop();
    if ( !QVariant(_result).toBool() ) {
        QString msg = data.join("\n");
        QString time = QTime::currentTime().toString();
        QString errorMsg = QString("<b>%1 %2:</b><br>%3").arg(time).arg(method).arg(msg);
        docContent->appendErrorMsg(errorMsg);
        return;
    };
    if ( method == "getVirtNetList" ) {
        if ( data.count() > virtNetModel->virtNetDataList.count() ) {
            int _diff = data.count() - virtNetModel->virtNetDataList.count();
            for ( int i = 0; i<_diff; i++ ) {
                virtNetModel->insertRow(1);
                //qDebug()<<i<<"insert";
            };
        };
        if ( virtNetModel->virtNetDataList.count() > data.count() ) {
            int _diff = virtNetModel->virtNetDataList.count() - data.count();
            for ( int i = 0; i<_diff; i++ ) {
                virtNetModel->removeRow(0);
                //qDebug()<<i<<"remove";
            };
        };
        int i = 0;
        foreach (QString _data, data) {
            QStringList chain = _data.split(" ");
            if (chain.isEmpty()) continue;
            int count = chain.size();
            for (int j=0; j<count; j++) {
                virtNetModel->setData(virtNetModel->index(i,j), chain.at(j), Qt::EditRole);
            };
            i++;
        };
    };
}
void VirtNetControl::warningShow(QString title, QString msg)
{
    QString time = QTime::currentTime().toString();
    QString errorMsg = QString("<b>%1 %2:</b><br>%3").arg(time).arg(title).arg(msg);
    docContent->appendErrorMsg(errorMsg);
}
void VirtNetControl::mouseMoveEvent(QMouseEvent *ev)
{
    if ( ev->type()==QEvent::MouseMove ) doc->showDoc();
}
void VirtNetControl::startUpdateTimer()
{
    docContent->startUpdateTimer();
}

void VirtNetControl::networkClicked(const QPoint &p)
{
    //qDebug()<<"custom Menu request";
    QModelIndex idx = virtNetList->indexAt(p);
    if ( idx.isValid() ) {
        //qDebug()<<virtNetModel->virtNetDataList.at(idx.row())->getName();
        QStringList params;
        params<<virtNetModel->virtNetDataList.at(idx.row())->getName();
        params<<virtNetModel->virtNetDataList.at(idx.row())->getState();
        params<<virtNetModel->virtNetDataList.at(idx.row())->getAutostart();
        params<<virtNetModel->virtNetDataList.at(idx.row())->getPersistent();
        VirtNetControlMenu *netControlMenu = new VirtNetControlMenu(this, params);
        connect(netControlMenu, SIGNAL(execMethod(const QStringList&)), this, SLOT(actionActivated(const QStringList&)));
        netControlMenu->move(QCursor::pos());
        netControlMenu->exec();
        disconnect(netControlMenu, SIGNAL(execMethod(const QStringList&)), this, SLOT(actionActivated(const QStringList&)));
        netControlMenu->deleteLater();
    } else {
        virtNetList->clearSelection();
    }
}
void VirtNetControl::networkDoubleClicked(const QModelIndex &index)
{
    if ( index.isValid() ) {
        qDebug()<<virtNetModel->virtNetDataList.at(index.row())->getName();
    }
}
void VirtNetControl::appendNetworkName(const QStringList &l)
{
    QStringList params;
    QModelIndex idx = virtNetList->currentIndex();
    if ( idx.isValid() ) {
        QString networkName = virtNetModel->virtNetDataList.at(idx.row())->getName();
        params = l;
        params.append(networkName);
        if ( l.first()=="setAutostartVirtNetwork" ) {
            /* set the opposite value */
            QString autostartState =
                (virtNetModel->virtNetDataList.at(idx.row())->getAutostart()=="yes")
                 ? "0" : "1";
            params.append(autostartState);
        };
        actionActivated(params);
    }
}
